@@ -50,6 +50,12 @@ class EntityManager(flow.Object):
             'asset types of this project.'
         ))
     
+    def get_asset_library_collection(self):
+        raise NotImplementedError((
+            'Must return the collection containing all the '
+            'asset libraries of this project.'
+        ))
+    
     def get_task_collection(self):
         raise NotImplementedError((
             'Must return the collection containing all the '
@@ -62,6 +68,12 @@ class EntityManager(flow.Object):
             'files of this project.'
         ))
     
+    def get_file_ref_collection(self):
+        raise NotImplementedError((
+            'Must return the collection containing all the '
+            'file references of this project.'
+        ))
+    
     def get_revision_collection(self):
         raise NotImplementedError((
             'Must return the collection containing all the '
