@@ -5,7 +5,7 @@ from fnmatch import fnmatch
 
 from kabaret import flow
 from kabaret.flow_contextual_dict import ContextualView, get_contextual_dict
-from kabaret.flow_entities.entities import Entity
+from kabaret.flow_entities.entities import Entity, Property
 
 from ..utils.kabaret.flow_entities.entities import EntityView
 from ..utils.flow.file_processing import ProcessSequenceFiles
@@ -738,7 +738,8 @@ class Sequence(KitsuSequence):
     settings = flow.Child(ContextualView).ui(hidden=True)
     description = flow.Param("")
     toggle_bookmark = flow.Child(ToggleBookmarkAction)
-    shots = flow.Child(Shots).ui(expanded=True)
+    shots = flow.Child(Shots).ui(expanded=True,
+                                 default_height=600)
     
     render_playblasts = flow.Child(RenderSequencePlayblasts)
     upload_playblasts = flow.Child(UploadSequencePlayblastsToKitsu)
@@ -794,11 +795,17 @@ class Film(Entity):
 
     ICON = ('icons.flow', 'film')
 
+    code = Property().ui(hidden=True)
+    display_name = Property().ui(hidden=True)
+
     sequences = flow.Child(SequenceCollection).ui(
         expanded=True,
-        show_filter=True
+        show_filter=True,
+        default_height=600
     )
 
+    settings = flow.Child(ContextualView).ui(hidden=True)
+
     @classmethod
     def get_source_display(cls, oid):
         split = oid.split('/')
@@ -806,7 +813,11 @@ class Film(Entity):
     
     def get_default_contextual_edits(self, context_name):
         if context_name == 'settings':
-            return dict(film=self.name())
+            return dict(
+                film=self.name(),
+                film_code=self.code.get(),
+                film_display_name=self.display_name.get()
+            )
 
 
 class FilmCollection(EntityView):
@@ -825,3 +836,6 @@ class FilmCollection(EntityView):
     def collection_name(self):
         mgr = self.root().project().get_entity_manager()
         return mgr.get_film_collection().collection_name()
+    
+    def _fill_row_cells(self, row, item):
+        row['Name'] = item.display_name.get() or item.name()
