@@ -1,8 +1,9 @@
 import pprint
 import gazu
 import os
-import fnmatch
+import re
 import json
+import time
 from collections import defaultdict
 
 from kabaret import flow
@@ -19,7 +20,6 @@ class KitsuAPIWrapper(flow.Object):
 
     _server_url = flow.Param("")
     _config = flow.Parent()
-    _user_login_per_mapped_name = flow.HashParam()
 
     def set_host(self, url):
         gazu.client.set_host(url)
@@ -33,8 +33,7 @@ class KitsuAPIWrapper(flow.Object):
     def get_server_url(self):
         return self._server_url.get()
 
-    def log_in(self, user_name, password):
-        login = self.get_user_login(user_name)
+    def log_in(self, login, password):
         success = False
 
         if login is not None:
@@ -44,7 +43,7 @@ class KitsuAPIWrapper(flow.Object):
                 gazu.exception.AuthFailedException,
                 gazu.exception.ServerErrorException,
             ):
-                pass
+                return success
             else:
                 success = True
 
@@ -61,18 +60,6 @@ class KitsuAPIWrapper(flow.Object):
 
     def host_is_valid(self):
         return gazu.client.host_is_valid()
-    
-    def get_user_login(self, user_mapped_name):
-        kitsu_login = None
-        if self._user_login_per_mapped_name.has_key(user_mapped_name):
-            kitsu_login = self._user_login_per_mapped_name.get_key(user_mapped_name)
-        
-        return kitsu_login
-    
-    def set_user_login(self, user_mapped_name, kitsu_login):
-        self._user_login_per_mapped_name.set_key(
-            user_mapped_name, kitsu_login
-        )
 
     def current_user_logged_in(self):
         """
@@ -120,12 +107,23 @@ class KitsuAPIWrapper(flow.Object):
         else:
             return data['fps']
 
+    def get_project_type(self):
+        import requests
+        try:
+            data = gazu.project.get_project_by_name(
+                self._config.project_name.get()
+            )
+        except requests.exceptions.ConnectionError:
+            return None
+        else:
+            return data['production_type']
+
     def get_entity_type_data(self, entity_type_id):
         return gazu.entity.get_entity_type(entity_type_id)
    
-    def get_shot_data(self, name, sequence):
+    def get_shot_data(self, name, sequence, episode_name=None):
         if isinstance(sequence, str):
-            sequence = self.get_sequence_data(sequence)
+            sequence = self.get_sequence_data(sequence, episode_name)
 
         if not sequence:
             return None
@@ -135,15 +133,37 @@ class KitsuAPIWrapper(flow.Object):
             name
         )
 
-    def get_shots_data(self, sequence):
+    def get_shot_duration(self, name, sequence, episode_name=None):
+        shot_data = self.get_shot_data(name, sequence, episode_name)
+        return shot_data["nb_frames"] if shot_data["nb_frames"] is not None else 0
+
+    def get_shots_data(self, sequence, episode_name=None):
         if isinstance(sequence, str):
-            sequence = self.get_sequence_data(sequence)
+            sequence = self.get_sequence_data(sequence, episode_name)
 
         if not sequence:
             return None
         
         return gazu.shot.all_shots_for_sequence(sequence)
     
+    def create_shot(self, sequence_name, shot_name, task_status=None):
+        '''
+        Creates a shot in the Kitsu project with all its tasks with the status `Todo`.
+        '''
+        sequence = self.get_sequence_data(sequence_name)
+        
+        if sequence is None:
+            sequence = self.create_sequence(sequence_name)
+        
+        shot = gazu.shot.new_shot(self._config.project_id.get(), sequence, shot_name)
+
+        if task_status:
+            task_status = self.get_task_status(short_name=task_status)
+
+        for task_type in gazu.task.all_task_types_for_project(self._config.project_id.get()):
+            if task_type['for_entity'] == 'Shot':
+                gazu.task.new_task(shot, task_type, task_status=task_status)
+    
     def get_shots(self, task_status_filters=None):
         if task_status_filters is not None:
             shots = []
@@ -189,55 +209,100 @@ class KitsuAPIWrapper(flow.Object):
         
         return shots
     
-    def get_shot_casting(self, shot, sequence=None):
+    def get_shot_casting(self, shot, sequence=None, episode_name=None):
         if isinstance(shot, str):
-            shot = self.get_shot_data(shot, sequence)
+            shot = self.get_shot_data(shot, sequence, episode_name)
 
         if not shot:
             return None
 
         return gazu.casting.get_shot_casting(shot)
     
-    def get_sequence_data(self, name):
+    def get_sequence_data(self, name, episode_name=None):
+        episode = None
+        if episode_name is not None:
+            episode = self.get_episode_data(episode_name)
+        
         return gazu.shot.get_sequence_by_name(
             self._config.project_id.get(),
-            name
+            name,
+            episode=episode
         )
 
-    def get_sequences_data(self):
-        return gazu.shot.all_sequences_for_project(
-            self._config.project_id.get()
-        )
+    def get_sequences_data(self, episode_name=None):
+        episode = None
+        if episode_name is not None:
+            episode = self.get_episode_data(episode_name)
+        
+        if episode is not None:
+            return gazu.shot.all_sequences_for_episode(episode)
+        else:
+            return gazu.shot.all_sequences_for_project(
+                self._config.project_id.get()
+            )
     
-    def get_sequence_casting(self, sequence):
+    def get_sequence_casting(self, sequence, episode_name=None):
         if isinstance(sequence, str):
-            sequence = self.get_sequence_data(sequence)
+            sequence = self.get_sequence_data(sequence, episode_name)
 
         if not sequence:
             return None
 
         return gazu.casting.get_sequence_casting(sequence)
     
+    def get_episode_data(self, name):
+        return gazu.shot.get_episode_by_name(
+            self._config.project_id.get(),
+            name
+        )
+    
+    def get_episodes_data(self):
+        return gazu.shot.all_episodes_for_project(
+            self._config.project_id.get()
+        )
+    
     def get_asset_data(self, name):
         return gazu.asset.get_asset_by_name(
             self._config.project_id.get(),
             name
         )
 
-    def get_assets_data(self, asset_type=None):
-        if not asset_type:
-            return gazu.asset.all_assets_for_project(self._config.project_id.get())
-
-        if isinstance(asset_type, str):
-            asset_type = self.get_asset_type_data(asset_type)
+    def get_assets_data(self, asset_type=None, episode_name=None):
+        '''
+        Returns a list of Kitsu asset data.
 
+        The result may be filtered by asset type and episode. If
+        the `episode_name` is "default_episode", returns the assets
+        belonging to the Kitsu default episode "Main Pack".
+        '''
         if not asset_type:
-            return None
+            assets_data = gazu.asset.all_assets_for_project(self._config.project_id.get())
+        else:
+            if isinstance(asset_type, str):
+                asset_type = self.get_asset_type_data(asset_type)
+
+            if not asset_type:
+                return None
+            
+            assets_data = gazu.asset.all_assets_for_project_and_type(
+                self._config.project_id.get(),
+                asset_type
+            )
         
-        return gazu.asset.all_assets_for_project_and_type(
-            self._config.project_id.get(),
-            asset_type
-        )
+        if episode_name is not None:
+            # FIXME: find a prettier way to handle Kitsu 'Main Pack' default episode
+            if episode_name == 'default_episode':
+                assets_data = [
+                    a for a in assets_data
+                    if a['source_id'] is None
+                ]
+            else:
+                assets_data = [
+                    a for a in assets_data
+                    if a['source_id'] is not None and episode_name == gazu.shot.get_episode(a['source_id']).get('name')
+                ]
+            
+        return assets_data
     
     def get_asset_type(self, asset):
         if isinstance(asset, str):
@@ -298,17 +363,17 @@ class KitsuAPIWrapper(flow.Object):
     def get_all_comments_for_task(self, task_id):
         return gazu.task.all_comments_for_task(task_id)
 
-    def get_shot_task(self, sequence_name, shot_name, task_type_name):
+    def get_shot_task(self, sequence_name, shot_name, task_type_name, episode_name=None):
         task = None
-        shot = self.get_shot_data(shot_name, sequence_name)
+        shot = self.get_shot_data(shot_name, sequence_name, episode_name)
 
         if shot is not None:
             task = self.get_task(shot, task_type_name)
         
         return task
     
-    def get_shot_task_status_name(self, sequence_name, shot_name, task_type_name):
-        task = self.get_shot_task(sequence_name, shot_name, task_type_name)
+    def get_shot_task_status_name(self, sequence_name, shot_name, task_type_name, episode_name=None):
+        task = self.get_shot_task(sequence_name, shot_name, task_type_name, episode_name)
         status = None
         
         if task is not None:
@@ -337,32 +402,38 @@ class KitsuAPIWrapper(flow.Object):
         elif entity_type == 'Asset':
             return [t['name'] for t in types if t['for_entity'] == 'Asset']
         return [t['for_entity']+' - '+t['name'] for t in types]
-    
+
     def get_user(self, project_user_name=None):
         if project_user_name is None:
             project_user_name = self.root().project().get_user_name()
-        
+
         user = self.root().project().get_user(project_user_name)
-        
+
         if user is None:
-            self.root().session().log_error((
-                f"No user '{project_user_name}' registered in this project"
-            ))
-            return None
-        
-        user_kitsu_id = self.get_user_login(project_user_name)
-        
-        if user_kitsu_id is None:
-            self.root().session().log_error((
-                f"No Kitsu login found for user '{project_user_name}'"
-            ))
+            self.root().session().log_error(
+                (f"No user '{project_user_name}' registered in this project")
+            )
             return None
 
-        kitsu_user = gazu.person.get_person_by_desktop_login(user_kitsu_id)
-        
+        logins = user.login.get()
+        kitsu_user = None
+        for login in logins:
+            if "@" in login:
+                kitsu_user = gazu.person.get_person_by_email(login)
+            else:
+                kitsu_user = gazu.person.get_person_by_desktop_login(login)
+
+            if not kitsu_user:
+                continue
+            else:
+                break
+
         if kitsu_user is None:
-            kitsu_user = gazu.person.get_person_by_email(user_kitsu_id)
-        
+            self.root().session().log_error(
+                (f"No Kitsu login found for user '{project_user_name}'")
+            )
+            return None
+
         return kitsu_user
 
     def get_users(self):
@@ -372,7 +443,8 @@ class KitsuAPIWrapper(flow.Object):
         user_names = []
         for user_id in data['team']:
             user_dict = gazu.person.get_person(user_id)
-            user_names.append(user_dict['desktop_login'])
+            if user_dict.get('desktop_login', None) is not None:
+                user_names.append(user_dict['desktop_login'])
         return sorted(user_names)
     
     def user_is_assigned(self, kitsu_user, kitsu_task):
@@ -405,8 +477,8 @@ class KitsuAPIWrapper(flow.Object):
             if files:
                 gazu.task.add_attachment_files_to_comment(task_id, comment, files)
     
-    def set_shot_task_status(self, sequence_name, shot_name, task_type_name, task_status_name, comment=''):
-        task = self.get_shot_task(sequence_name, shot_name, task_type_name)
+    def set_shot_task_status(self, sequence_name, shot_name, task_type_name, task_status_name, comment='', episode_name=None):
+        task = self.get_shot_task(sequence_name, shot_name, task_type_name, episode_name)
 
         if task is not None:
             status = gazu.task.get_task_status_by_name(task_status_name)
@@ -414,8 +486,8 @@ class KitsuAPIWrapper(flow.Object):
             if status is not None:
                 gazu.task.add_comment(task, status, comment=comment)
     
-    def upload_shot_preview(self, sequence_name, shot_name, task_type_name, task_status_name, file_path, comment="", user_name=None):
-        shot = self.get_shot_data(shot_name, sequence_name)
+    def upload_shot_preview(self, sequence_name, shot_name, task_type_name, task_status_name, file_path, comment="", episode_name=None, user_name=None):
+        shot = self.get_shot_data(shot_name, sequence_name, episode_name)
         result = False
         
         if shot is not None:
@@ -476,11 +548,80 @@ class KitsuAPIWrapper(flow.Object):
         return True
 
 
+class AddKitsuTaskTypeFile(flow.Action):
+
+    oid_patterns = flow.Param('.*mov$')
+    task_patterns = flow.Param(['.*'])
+
+    _map = flow.Parent()
+
+    def needs_dialog(self):
+        return True
+
+    def allow_context(self, context):
+        return context
+
+    def get_buttons(self):
+        return ['Create', 'Cancel']
+    
+    def run(self, button):
+        if button == 'Cancel':
+            return
+        
+        task_type_file = self._map.add(f'task_type_file_{str(int(time.time()))}')
+        task_type_file.oid_patterns.set(self.oid_patterns.get())
+        task_type_file.task_patterns.set(self.task_patterns.get())
+        self._map.touch()
+
+
+class RemoveKitsuTaskTypeFile(flow.Action):
+
+    ICON = ('icons.gui', 'remove-symbol')
+
+    _task_type_file = flow.Parent()
+    _map = flow.Parent(2)
+
+    def needs_dialog(self):
+        return False
+
+    def allow_context(self, context):
+        return context
+
+    def run(self, button):
+        _map = self._map
+        _map.remove(self._task_type_file.name())
+        _map.touch()
+
+
+class KitsuTaskTypeFile(flow.Object):
+
+    oid_patterns = flow.Param()
+    task_patterns = flow.Param()
+
+    delete = flow.Child(RemoveKitsuTaskTypeFile)
+
+
+class KitsuTaskTypeFiles(flow.Map):
+
+    add_task_type_file = flow.Child(AddKitsuTaskTypeFile)
+
+    @classmethod
+    def mapped_type(cls):
+        return KitsuTaskTypeFile
+
+    def columns(self):
+        return ["OID Patterns", "Task Types"]
+
+    def _fill_row_cells(self, row, item):
+        row["OID Patterns"] = item.oid_patterns.get()
+        row["Task Types"] = item.task_patterns.get()
+
+
 class KitsuBindings(flow.Object):
 
     asset_types = flow.HashParam()
     asset_families = flow.HashParam()
-    task_type_files = flow.DictParam({})
+    task_type_files = flow.Child(KitsuTaskTypeFiles)
 
     def _kitsu_api(self):
         return self.root().project().kitsu_api()
@@ -547,25 +688,41 @@ class KitsuBindings(flow.Object):
         
         return self._kitsu_api().get_task(entity, task_type_name)
     
-    def get_task_types(self, file_name):
+    def get_task_types(self, oid):
         task_types = []
+        kitsu_config = self.root().project().kitsu_config()
 
-        for pattern, type_names in self.task_type_files.get().items():
-            if not isinstance(type_names, list):
-                type_names = [type_names]
+        for task_type_file in self.task_type_files.mapped_items():
+            oid_pattern = task_type_file.oid_patterns.get()
+            task_patterns = task_type_file.task_patterns.get()
             
-            if fnmatch.fnmatch(pattern, file_name):
-                task_types += type_names
+            if not isinstance(task_patterns, list):
+                task_patterns = [task_patterns]
+            
+            # Match file name
+            if re.search(oid_pattern, oid):
+                # Fetch every task that matches
+                task_types = [
+                    t['name']
+                    for t in gazu.task.all_task_types_for_project(kitsu_config.project_id.get())
+                    for pattern in task_patterns
+                    if re.search(pattern, t['name'])
+                ]
         
         return task_types
     
     def get_entity_data(self, contextual_settings):
         kitsu_api = self.root().project().kitsu_api()
         entities_data = {
-            'shot': ['shot', 'sequence'],
+            'shot': ['shot', 'sequence', 'episode'],
             'asset': ['asset'],
         }
         
+        # Remove episode key if kitsu project is not a TV Show
+        project_type = kitsu_api.get_project_type()
+        if project_type != 'tvshow':
+            entities_data['shot'].remove('episode')
+        
         entity_data = {}
         
         for entity_type, entity_keys in entities_data.items():
@@ -581,15 +738,31 @@ class KitsuBindings(flow.Object):
         return None
     
     def get_kitsu_entity(self, entity_data):
+        if entity_data is None:
+            return None
+
         entity_type = entity_data['entity_type']
         
         if entity_type == 'shot':
-            return self._kitsu_api().get_shot_data(entity_data['shot'], entity_data['sequence'])
+            return self._kitsu_api().get_shot_data(entity_data['shot'], entity_data['sequence'], entity_data.get('episode'))
         elif entity_type == 'asset':
             return self._kitsu_api().get_asset_data(entity_data['asset'])
         else:
             return None
 
+    def set_task_type(self, file_name, task_type):
+        existing = [t for t in self.task_type_files.mapped_items() if file_name in t.oid_patterns.get()]
+        if len(existing) > 0:
+            existing = existing[0].task_patterns.set(task_type)
+        else:
+            new = self.task_type_files.add(f"task_type_file_{str(int(time.time()))}")
+            new.oid_patterns.set(file_name)
+            new.task_patterns.set(task_type)
+        
+        self.task_type_files.touch()
+        
+        # return self.task_type_files.get()
+
 
 class ChangeKitsuUserLogin(flow.Action):
 
@@ -613,37 +786,6 @@ class ChangeKitsuUserLogin(flow.Action):
         self._map.touch()
 
 
-class KitsuUser(flow.SessionObject):
-    
-    edit = flow.Child(ChangeKitsuUserLogin)
-
-    def get_login(self):
-        return self.root().project().kitsu_api().get_user_login(self.name())
-    
-    def set_login(self, login):
-        self.root().project().kitsu_api().set_user_login(self.name(), login)
-
-
-class KitsuUsers(flow.DynamicMap):
-
-    @classmethod
-    def mapped_type(cls):
-        return KitsuUser
-
-    def mapped_names(self, page_num=0, page_size=None):
-        return self.root().project().get_users().mapped_names()
-    
-    def columns(self):
-        return ['Name', 'Login']
-    
-    def _fill_row_cells(self, row, item):
-        row['Name'] = item.name()
-        row['Login'] = item.get_login()
-    
-    def _fill_row_style(self, style, item, row):
-        style['activate_oid'] = item.edit.oid()
-
-
 class EditKitsuSubtask(flow.Action):
 
     kitsu_task = flow.SessionParam('')
@@ -714,12 +856,24 @@ class KitsuTasks(flow.DynamicMap):
         return tm.default_tasks.mapped_names()
 
 
+class KitsuTaskAvailableStatutes(flow.values.MultiChoiceValue):
+
+    DEFAULT_EDITOR = 'multichoice'
+
+    def choices(self):
+        return self.root().project().kitsu_api().get_task_statutes(short_name=True)
+
+
 class KitsuTaskStatus(PresetChoiceValue):
     
     DEFAULT_EDITOR = 'choice'
     
     def choices(self):
-        return self.root().project().kitsu_api().get_task_statutes()
+        status = self.root().project().kitsu_config().task_statutes.get()
+        self.names_dict={}
+        for s in status:
+            self.names_dict[s.upper()] = s
+        return self.names_dict.keys()
 
 
 class UpdateKitsuSettings(flow.Action):
