@@ -43,7 +43,7 @@ class CreateGenericAction(flow.Action):
 
     def run(self, button):
         if self.entity_name.get() == "":
-            print('Action not executed as you have to provide an entity name')
+            self.root().session().log_warning('[Create Action] Not executed as you have to provide an entity name')
             return
         self._map.add(self.entity_name.get())
         self._map.touch()
@@ -102,6 +102,9 @@ class SimpleCreateAction(CreateGenericItemAction):
     def needs_dialog(self):
         return True
 
+    def allow_context(self, context):
+        return context
+
     def run(self, button):
         if button == "Cancel":
             return
