@@ -2,17 +2,19 @@ import os
 import sys
 import gazu
 import time
+import timeago
 from datetime import datetime, timedelta
 import subprocess
 import platform
 from minio import Minio
+from ..scripts.minio_progress import Progress
 import zipfile
 import fnmatch
 import re
 import traceback
 import tempfile
 import uuid
-import pathlib
+from pathlib import Path, PurePath
 
 from kabaret import flow
 from kabaret.flow_contextual_dict import ContextualView, get_contextual_dict
@@ -25,6 +27,7 @@ from ..utils.kabaret.flow_entities.entities import CustomEntity, CustomEntityCol
 from .dependency import get_dependencies
 from .maputils import ItemMap, CreateGenericAction, ClearMapAction, RemoveGenericAction, SimpleCreateAction
 from .runners import Runner, PythonRunner
+from .users import PresetSessionValue
 
 
 class SiteTypeChoiceValue(PropertyChoiceValue):
@@ -66,43 +69,111 @@ class LoadType(PropertyValue):
 class JobStatus(PropertyValue):
 
     DEFAULT_EDITOR = 'choice'
-    CHOICES = ['WFA', 'WAITING', 'PROCESSED', 'ERROR', 'PAUSE', 'DONE']
+    CHOICES = ['WFA', 'WAITING', 'PROCESSING', 'PROCESSED', 'ERROR', 'PAUSE', 'DONE']
 
     def choices(self):
         return self.CHOICES
 
 
 class ResetJob(flow.Action):
+    """Reset one job.
 
-    ICON = ('icons.libreflow', 'reset')
+    Return an error if the file doesn't exist on the exchange server
+
+    """
+    # ICON = ('icons.libreflow', 'reset')
 
     _job = flow.Parent()
     _jobs = flow.Parent(2)
 
     def needs_dialog(self):
-        o = self.root().get_object(self._job.emitter_oid.get())
-        return self._job.type.get() == 'Upload' and o.get_sync_status(exchange=True) == 'Available'
-    
+        """Indicate whether this action requires a dialog to be displayed.
+        
+        Returns:
+            bool: True if job type equal upload, revision status equal Available and revision is valid.
+            
+        """
+        rev = self._job.get_related_object()
+        return self._job.type.get() == 'Upload' \
+            and rev is not None \
+            and rev.get_sync_status(exchange=True) == 'Available'
+
     def allow_context(self, context):
+        """Check whether the given context is valid for running the action.
+        
+        Args:
+            context: Context object, usually representing the current project/task.
+        
+        Returns:
+            bool: True if context valid and job status equal ERROR, otherwhise False.
+            
+        """
         return context and self._job.status.get() == 'ERROR'
-    
+
     def get_buttons(self):
+        """Return the buttons displayed in the dialog.
+        
+        Returns:
+            list[str]: list of button labels, typically ['Confirm', 'Cancel'].
+            
+        """
         self.message.set((
             '<h3>Revision already on the exchange server</h3>'
             'Reset upload job anyway ?'
         ))
         return ['Confirm', 'Cancel']
 
+    def _is_file_exchange_server(self, revision):
+        """Check if a file exist in the exchange server.
+
+        Args:
+            revision (Revision): revision of the file
+
+        Returns:
+            bool: True if file exist otherwise False.
+
+        """
+        exchange = self.root().project().get_exchange_site()
+        bucket_name = exchange.bucket_name.get()
+        minio_client = exchange.sync_manager._ensure_client()
+        path = Path(revision.get_path(relative=True)).as_posix()
+
+        if os.path.isdir(path):
+            path = path + ".zip"
+
+        # Check if object exists
+        try:
+            minio_client.stat_object(bucket_name, path)
+        except Exception as error_msg:
+            self.root().session().log_error(error_msg)
+            self.root().session().log_info('DO NOT EXIST')
+            revision.set_sync_status('NotAvailable', exchange=True)
+            return False
+
+        return True
+
     def run(self, button):
-        if button == 'Cancel':
-            return
-        
-        self._job.status.set('WAITING')
-        self._job.log.set('?')
-        self._job.on_submitted()
-        self._jobs.incr_waiting_count()
-        o = self.root().get_object(self._job.emitter_oid.get())
-        o.set_sync_status('NotAvailable', exchange=True)
+        """Execute the render action.
+
+        Args:
+            button (str): The label of the button pressed by the user (e.g., 'Confirm' or 'Cancel').
+
+        Returns:
+            Any: the result of the parent run method if executed, or None if canceled.
+
+        """
+        if button == "Cancel":
+            return None
+
+        self._job.status.set("WAITING")
+        self._job.log.set("?")
+        rev = self._job.get_related_object()
+        if rev is None:
+            return self.get_result()
+
+        self._is_file_exchange_server(rev)
+
+        return None
 
 
 class ResetJobs(flow.Action):
@@ -118,7 +189,7 @@ class ResetJobs(flow.Action):
         if button == 'Cancel':
             return
         
-        for j in self._jobs.jobs(status='ERROR'):
+        for j in self._jobs.jobs(status=['ERROR']):
             j.status.set('WAITING')
         
         self._jobs.touch()
@@ -130,6 +201,9 @@ class ShowRevisionInHistory(flow.Action):
 
     _job = flow.Parent()
 
+    def allow_context(self,context):
+        return context
+
     def needs_dialog(self):
         return False
 
@@ -153,22 +227,27 @@ class Job(Entity):
     requested_by_user   = Property()
     requested_by_studio = Property()
     log                 = Property().ui(editor='textarea')
+    file_size           = Property()
 
     reset = flow.Child(ResetJob)
     show = flow.Child(ShowRevisionInHistory).ui(label='Show in history')
+    _map = flow.Parent()
 
     def __repr__(self):
+        if self._map._document_cache is None:
+            self._map.mapped_names()
+        name = self.name()
         job_repr = "%s(type=%s, status=%s, priority=%s, emitter=%s, date=%s, date_end=%s, is_archived=%s, requested_by_user=%s, requested_by_studio=%s)" % (
                 self.__class__.__name__,
-                self.type.get(),
-                self.status.get(),
-                self.priority.get(),
-                self.emitter_oid.get(),
-                self.date.get(),
-                self.date_end.get(),
-                self.is_archived.get(),
-                self.requested_by_user.get(),
-                self.requested_by_studio.get()
+                self._map._document_cache[name]['type'],
+                self._map._document_cache[name]['status'],
+                self._map._document_cache[name]['priority'],
+                self._map._document_cache[name]['emitter_oid'],
+                self._map._document_cache[name]['date'],
+                self._map._document_cache[name]['date_end'],
+                self._map._document_cache[name]['is_archived'],
+                self._map._document_cache[name]['requested_by_user'],
+                self._map._document_cache[name]['requested_by_studio']
             )
 
         return job_repr
@@ -201,7 +280,9 @@ class Job(Entity):
             o = self.root().get_object(
                 self.emitter_oid.get()
             )
-        except (ValueError, flow.exceptions.MissingRelationError):
+        except (ValueError, \
+            flow.exceptions.MissingRelationError, \
+            flow.exceptions.MappedNameError):
             o = None
         
         return o
@@ -219,12 +300,17 @@ class Job(Entity):
             return False
         
         o = self.get_related_object()
+        if o is None:
+            log_msg = "Revision does not exist"
+            self.status.set('ERROR')
+            self.log.set(log_msg)
+            self.root().session().log_warning(log_msg)
+            return False
 
         if self.type.get() == 'Download' and o.get_sync_status(exchange=True) != 'Available':
-            self.log.set((
-                'Revision is not yet available '
-                'on the exchange server'
-            ))
+            log_msg = f"Revision {o.oid()} not yet available on exchange server"
+            self.log.set(log_msg)
+            self.root().session().log_debug(log_msg)
             return False
         
         return True
@@ -259,6 +345,9 @@ class Job(Entity):
                 self.log.set('Revision is not ready to be synced')
                 return
         
+        if revision.file_size.get() is not None :
+            self.file_size.set(revision.file_size.get())
+        
         self.path.set(path)
     
     def on_processed(self):
@@ -270,9 +359,10 @@ class Job(Entity):
         revision statutes. The method assumes that
         `emitter_oid` references a valid revision object.
         '''
-        if self.status.get() != 'PROCESSED':
-            return
-        
+        self.status.set('PROCESSED')
+        self.date_end.set(time.time())
+
+        self.root().project().get_current_site().set_sync_date()
         revision = self.root().get_object(
             self.emitter_oid.get()
         )
@@ -283,33 +373,176 @@ class Job(Entity):
         revision.touch()
         revision._revisions.touch()
 
+class RemoveOutdatedJobs(flow.Action):
+
+    _map = flow.Parent()
+
+    def allow_context(self,context):
+        return context
+
+    def needs_dialog(self):
+        self.message.set(f'Removing outdated jobs...')
+        return True
+    
+    def get_buttons(self):
+
+        count=0
+
+        for job in self._map.mapped_items():
+
+            if job.status.get() == 'WAITING' or job.status.get() == 'ERROR':
+
+                self.root().session().log_info(f'[RemoveOutdatedJobs] Checking entity: {job.emitter_oid.get()}')
+
+                if self.root().session().cmds.Flow.exists(job.emitter_oid.get()):
+                
+                    rev = self.root().get_object(job.emitter_oid.get())
+                    file = self.root().get_object(re.search('.+(?=\/history)', job.emitter_oid.get()).group(0))
+                    latest_rev = file.get_head_revision()
+
+                    if (rev.get_sync_status(site_name=job.requested_by_studio.get()) == 'NotAvailable' 
+                        or rev.get_sync_status(site_name=job.requested_by_studio.get()) == 'Requested'):
+
+                        latest_rev.sync_statutes.touch()
+                        if latest_rev.get_sync_status() == 'Available':
+
+                            # if job.status.get() == 'ERROR' and 'empty revision' not in rev.comment.get() :
+                            #     continue
+
+                            self.root().session().log_info(f'[RemoveOutdatedJobs] Removing Job')
+                            count+=1
+                            self._map.delete_entities([job.name()])
+                    
+                    # elif file.file_type.get() == 'Outputs' and '_CYCLE' in job.emitter_oid.get():
+                    #     self.root().session().log_info(f'[RemoveOutdatedJobs] Removing Job')
+                    #     count+=1
+                    #     self._map.delete_entities([job.name()])
+
+                else : 
+                    self.root().session().log_info(f'[RemoveOutdatedJobs] Removing Job')
+                    count+=1
+                    self._map.delete_entities([job.name()])
+
+        self.message.set(f'Removed {count} jobs')
+        return ['OK']
+
+    def run(self,button):
+        if button == 'OK':
+            return
+
+class ClearQueueJobStatus(flow.values.ChoiceValue):
+
+    CHOICES = JobStatus.CHOICES + ["All"]
+
+
+class ClearQueueLoadType(flow.values.ChoiceValue):
+
+    CHOICES = LoadType.CHOICES + ["All"]
+
+
+class ClearQueueAction(flow.Action):
+
+    _site = flow.Parent(3)
+    status = flow.Param("PROCESSED", ClearQueueJobStatus)
+    type = flow.Param("All", ClearQueueLoadType)
+    emitted_before = flow.SessionParam((datetime.now() - timedelta(days=1)).timestamp()).ui(
+        editor='datetime'
+    )
+
+    def get_buttons(self):
+        self.message.set(
+            "<h2>Clear %s jobs</h2>" % self._site.name()
+        )
+        return ["Clear", "Cancel"]
+    
+    def run(self, button):
+        if button == 'Cancel':
+            return
+        
+        queue = self._site.get_queue()
+        status = self.status.get()
+        type = self.type.get()
+
+        if status == "All" and type == "All":
+            queue.clear()
+        else:
+            for job in queue.jobs(status=[self.status.get()]):
+                if job.date.get() < self.emitted_before.get():
+                    queue.delete_entities([job.name()])
+                    self.root().session().log_info(f'[Clear Queue] Deleted {job.oid()}')
+
+        queue.touch()
 
 class JobQueue(EntityCollection):
 
-    count_waiting = flow.IntParam(0)
     reset_jobs = flow.Child(ResetJobs).ui(label='Reset erroneous jobs')
+    remove_outdated_jobs = flow.Child(RemoveOutdatedJobs)
+    clear_queue = flow.Child(ClearQueueAction)
+
+    _queue = flow.Parent()
+    _site = flow.Parent(2)
 
     @classmethod
     def mapped_type(cls):
         return Job
     
+    def summary(self):
+        # Jobs count
+        all_jobs_count = f'{self._site.count_jobs()} Jobs in queue'
+        processed_count = f'<font color="#61f791">{self._site.count_jobs(status="PROCESSED")} PROCESSED</font>'
+        waiting_count = f'<font color="#EFDD5B">{self._site.count_jobs(status="WAITING")} WAITING</font>'
+        error_count = f'<font color="#ff5842">{self._site.count_jobs(status="ERROR")} ERROR</font>'
+
+        message = f'{all_jobs_count} / {processed_count} - {waiting_count} - {error_count}'
+
+        # Latest sync dates
+        if self._queue.last_auto_sync.get():
+            date = datetime.fromtimestamp(self._queue.last_auto_sync.get())
+            full_date = date.strftime('%Y-%m-%d %H:%M:%S')
+            nice_date = timeago.format(full_date, datetime.now())
+
+            message += f'<br>Last auto sync: {full_date} ({nice_date})</br>'
+
+        if self._queue.last_manual_sync.get():
+            date = datetime.fromtimestamp(self._queue.last_manual_sync.get())
+            full_date = date.strftime('%Y-%m-%d %H:%M:%S')
+            nice_date = timeago.format(full_date, datetime.now())
+
+            message += f'<br>Last manual sync: {full_date} ({nice_date})</br>'
+
+        return message
+
     def get_next_waiting_job(self):
         for job in reversed(self.mapped_items()):
             if job.status.get() == "WAITING":
                 return job
         
         return None
-    
-    def jobs(self, type=None, status=None):
-        jobs = self.mapped_items()
 
+    def list_users(self):
+        coll = self.get_entity_store().get_collection(self.collection_name())
+        cursor = coll.distinct('requested_by_user')
+        return list(cursor)
+
+    def jobs(self, to_dict=False, type=None, status=None, user=None):
+        if type is None and status is None and to_dict is False:
+            return self.mapped_items()
+
+        job_filter = {}
+        coll = self.get_entity_store().get_collection(self.collection_name())
         if type is not None:
-            jobs = [j for j in jobs if j.type.get() == type]
+            job_filter["type"] = {"$in": type}
         if status is not None:
-            jobs = [j for j in jobs if j.status.get() == status]
-        
+            job_filter["status"] = {"$in": status}
+        if user is not None:
+            job_filter["requested_by_user"] = {"$in": user}
+        cursor = coll.find(job_filter)
+        if to_dict:
+            return list(cursor)
+
+        jobs = [self.get_mapped(d["name"]) for d in cursor]
         return jobs
-    
+
     def count(self, type=None, status=None):
         job_filter = {}
 
@@ -322,18 +555,6 @@ class JobQueue(EntityCollection):
 
         return c.count_documents(job_filter)
     
-    def update_waiting_count(self, count=None):
-        if count is None:
-            count = len(self.jobs(status='WAITING'))
-        
-        self.count_waiting.set(count)
-    
-    def incr_waiting_count(self, by=1):
-        self.count_waiting.incr(by)
-
-    def decr_waiting_count(self, by=1):
-        self.count_waiting.decr(by)
-    
     def submit_job(self,
             emitter_oid,
             user,
@@ -350,29 +571,25 @@ class JobQueue(EntityCollection):
             job_type,
             time.time()
         )
-        self.ensure_exist([name])
+        c = self.get_entity_store().get_collection(
+            self.collection_name())
+        c.insert_one({
+            "name": name,
+            "type": job_type,
+            "status": init_status,
+            "priority": priority,
+            "emitter_oid": emitter_oid,
+            "date": time.time(),
+            "date_end": date_end,
+            "requested_by_user": user,
+            "requested_by_studio": studio,
+            "is_archived": False,
+            "log": '?',
+        })
         self._document_cache = None
-
         job = self.get_mapped(name)
-        job.type.set(job_type)
-        job.status.set(init_status)
-        job.priority.set(priority)
-        job.emitter_oid.set(emitter_oid)
-        job.date.set(time.time())
-        job.date_end.set(date_end)
-        job.requested_by_user.set(user)
-        job.requested_by_studio.set(studio)
-        job.is_archived.set(False)
-        job.log.set('?')
-
-        job.on_submitted()
-
-        self.incr_waiting_count()
-
         self.root().session().log_info("Submitted job %s" % job)
-
         self.touch()
-
         return job
     
     def columns(self):
@@ -401,10 +618,6 @@ class JobQueue(EntityCollection):
 
 class ProcessJobs(flow.Action):
 
-    def __init__(self, parent, name):
-        super(ProcessJobs, self).__init__(parent, name)
-        self._job_count = {}
-
     def needs_dialog(self):
         return False
 
@@ -417,27 +630,10 @@ class ProcessJobs(flow.Action):
         current_site = self.root().project().get_current_site()
         return current_site.get_jobs()
     
-    def _compute_job_count(self):
-        count = dict.fromkeys(JobStatus.CHOICES, 0)
-
-        for j in self.root().project().get_current_site().get_jobs():
-            count[j.status.get()] += 1
-        
-        return count
-    
     def run(self, button):
-        self._job_count = self._compute_job_count()
-
         for job in self._get_jobs():
             self.root().session().log_info("Processing job %s" % job)
-            
-            self._job_count[job.status.get()] -= 1
             self.process(job)
-            self._job_count[job.status.get()] += 1
-        
-        current_site = self.root().project().get_current_site()
-        current_site.queue.job_list.update_waiting_count(self._job_count['WAITING'])
-
         # Refresh project's sync section UI
         self.root().project().synchronization.touch()
 
@@ -482,7 +678,8 @@ class MinioUploadFile(flow.Object):
         minioClient.fput_object(
             exchange_site.bucket_name.get(),
             server_path,
-            local_path
+            local_path,
+            progress=Progress()
         )
 
 
@@ -513,7 +710,8 @@ class MinioDownloadFile(flow.Object):
             exchange_site.bucket_name.get(),
             server_path,
             local_path,
-            tmp_file_path=tmp_path
+            tmp_file_path=tmp_path,
+            progress=Progress()
         )
 
 
@@ -525,7 +723,7 @@ class SyncManager(flow.Object):
         super(SyncManager, self).__init__(parent, name)
         self._client = None
 
-    def upload(self, server_path, local_path):
+    def upload(self, server_path, local_path, job_entity):
         zipped_folder = self._is_zipped_folder(
             server_path, local_path
         )
@@ -543,13 +741,14 @@ class SyncManager(flow.Object):
         self._ensure_client().fput_object(
             self._exchange.bucket_name.get(),
             server_path,
-            src_path
+            src_path,
+            progress=Progress(job_entity)
         )
 
         if zipped_folder:
             os.remove(src_path)
 
-    def download(self, server_path, local_path):
+    def download(self, server_path, local_path, job_entity):
         zipped_folder = self._is_zipped_folder(
             server_path, local_path
         )
@@ -564,7 +763,8 @@ class SyncManager(flow.Object):
         self._ensure_client().fget_object(
             self._exchange.bucket_name.get(),
             server_path,
-            dst_path
+            dst_path,
+            progress=Progress(job_entity)
         )
 
         if zipped_folder:
@@ -594,7 +794,7 @@ class SyncManager(flow.Object):
         return os.path.join(
             self.root().project().get_temp_folder(),
             (
-                pathlib.PurePath(server_path).stem
+                PurePath(server_path).stem
                 + f'-{str(uuid.uuid4())}.zip'
             )
         )
@@ -610,32 +810,32 @@ class Synchronize(ProcessJobs):
 
     def _get_jobs(self):
         return self.root().project().get_current_site().get_jobs(
-            status='WAITING'
+            status=['WAITING']
         )
 
     def process(self, job):
         if not job.check_valid_state():
             return
-        
+        job.on_submitted()
         try:
+            job.status.set('PROCESSING')
             sync_manager = self.root().project().get_exchange_site().sync_manager
         
             if job.type.get() == 'Upload':
                 sync_manager.upload(
                     job.get_server_path(),
-                    job.get_local_path()
+                    job.get_local_path(),
+                    job
                 )
             else:
                 sync_manager.download(
                     job.get_server_path(),
-                    job.get_local_path()
+                    job.get_local_path(),
+                    job
                 )
         except Exception as e:
             job.status.set('ERROR')
             job.log.set(traceback.format_exc())
-        else:
-            job.status.set('PROCESSED')
-            job.on_processed()
 
 
 class ActiveSiteChoiceValue(flow.values.SessionValue):
@@ -653,6 +853,15 @@ class ActiveSiteChoiceValue(flow.values.SessionValue):
         return self._choices.get()
 
 
+class ActiveSiteAutoSelectChoiceValue(ActiveSiteChoiceValue):
+    
+    def choices(self):
+        choices = super(ActiveSiteAutoSelectChoiceValue, self).choices()
+        choices = ['Auto select'] + choices.copy()
+        
+        return choices
+
+
 class ActiveSiteRevisionAvailableChoiceValue(flow.values.ChoiceValue):
     
     DEFAULT_EDITOR = 'choice'
@@ -687,18 +896,21 @@ class ActiveSiteRevisionAvailableChoiceValue(flow.values.ChoiceValue):
 
 class SiteSelection(flow.Object):
     
-    source_site = flow.Param(None, ActiveSiteChoiceValue).watched()
+    source_site = flow.Param(None, ActiveSiteAutoSelectChoiceValue).watched()
     target_site = flow.Param(None, ActiveSiteChoiceValue).watched()
-    auto_select_enabled = flow.SessionParam(False).watched().ui(
-        editor='bool',
-        hidden=True)
+
+    _action = flow.Parent()
+    _revision = flow.Parent(2)
     
     def child_value_changed(self, child_value):
-        if child_value in (self.source_site, self.target_site, self.auto_select_enabled):
+        if child_value in (self.source_site, self.target_site):
             self.touch()
     
     def summary(self):
-        if self.source_site.get() == self.target_site.get() and not self.auto_select_enabled.get():
+        if (self.source_site.get() == self.target_site.get()) or (
+            self.source_site.get() == "Auto select"
+            and self.target_site.get() == self._revision.site.get()
+        ):
             return '⚠️ Source and target sites can\'t be the same !'
         else:
             return ''
@@ -709,14 +921,13 @@ class RequestAs(flow.Action):
     _revision = flow.Parent()
     sites = flow.Child(SiteSelection).ui(expanded=True)
     priority = flow.IntParam(50)
-    include_dependencies = flow.SessionParam(True).ui(editor='bool')
     forced_upload = flow.SessionParam(False).ui(
         editor='bool',
         tooltip=('Ask the source site to upload the revision regardless'
-                 ' of its status on the exchange site.'))
+                ' of its status on the exchange site.'))
     
     def get_buttons(self):
-        self.sites.source_site.set(self.root().project().get_current_site().name())
+        self.sites.source_site.set(self.sites.source_site.choices()[0])
         target_site_choices = self.sites.target_site.choices()
         
         if not target_site_choices:
@@ -749,38 +960,45 @@ class RequestAs(flow.Action):
             and self.root().project().get_current_site().request_files_from_anywhere.get()
         )
 
+    def find_existing_job(self, job_queue, job_type, job_status, revision_oid):
+        coll = job_queue.get_entity_store().get_collection(
+            job_queue.collection_name())
+        result = coll.find({'emitter_oid': revision_oid,
+                            'type': job_type,
+                            'status': job_status})
+        return len(list(result)) != 0
+
     def run(self, button):
         if button == "Cancel":
             return
         
         source_site_name = self.get_source_site_name()
         target_site_name = self.get_target_site_name()
+        if source_site_name == target_site_name:
+            return self.get_result(close=False)
 
         # Get requesting and requested sites
         sites = self.root().project().get_working_sites()
         target_site = sites[target_site_name]
-        
-        # Add a download job for the requesting site
-        target_site.get_queue().submit_job(
-            job_type="Download",
-            init_status="WAITING",
-            emitter_oid=self._revision.oid(),
-            user=self.root().project().get_user_name(),
-            studio=target_site_name,
-            priority=self.priority.get(),
-        )
+        job_exists = self.find_existing_job(target_site.get_queue(), \
+            'Download', 'WAITING', self._revision.oid())
+        if not job_exists:
+            # Add a download job for the requesting site
+            target_site.get_queue().submit_job(
+                job_type="Download",
+                init_status="WAITING",
+                emitter_oid=self._revision.oid(),
+                user=self.root().project().get_user_name(),
+                studio=target_site_name,
+                priority=self.priority.get(),
+            )
+        else:
+            self.root().session().log_warning(
+                "Revision already requested for download in target site")
         self._revision.set_sync_status("Requested", site_name=target_site_name)
-        self._revision._revisions.touch()
-        
-        # Request dependencies if required
-        if self.include_dependencies.get():
-            request_deps = self._revision.request_dependencies
-            request_deps.target_site.set(target_site_name)
-            request_deps.predictive_only.set(False)
-            request_deps.run(None)
+        self._revision.touch()
         
         # Check if the version is not available on the exchange server
-        exchange_site = self.root().project().get_exchange_site()
         revision_status = self._revision.get_sync_status(exchange=True)
 
         if revision_status == "Available" and not self.forced_upload.get():
@@ -791,20 +1009,18 @@ class RequestAs(flow.Action):
             return self.get_result()
         
         # If enabled, automatically select source site
-        if self.sites.auto_select_enabled.get():
+        if self.sites.source_site.get() == "Auto select":
             source_site_name = self._revision.site.get()
         
         source_site = sites[source_site_name]
+        job_exists = self.find_existing_job(source_site.get_queue(), \
+            'Upload', 'WAITING', self._revision.oid())
+        if job_exists:
+            self.root().session().log_warning(
+                "Revision already requested for upload in source site")
+            self.root().project().synchronization.touch()
+            return self.get_result()
 
-        # Check if the source version upload is not already requested
-        for job in source_site.get_jobs(type="Upload", status="WAITING"):
-            if job.emitter_oid.get() == self._revision.oid():
-                self.root().session().log_warning(
-                    "Revision already requested for upload in source site"
-                )
-                self.root().project().synchronization.touch()
-                return self.get_result()
-        
         # Add an upload job for the requested site
         source_site.get_queue().submit_job(
             job_type="Upload",
@@ -823,12 +1039,16 @@ class Request(RequestAs):
 
     ICON = ('icons.libreflow', 'request')
 
-    _revision = flow.Parent()
-    priority = flow.IntParam(50)
     source_site = flow.Param(None, ActiveSiteRevisionAvailableChoiceValue)
-    
+    priority = flow.IntParam(50)
+    forced_upload = flow.SessionParam(False).ui(
+        editor='bool',
+        tooltip=('Ask the source site to upload the revision regardless'
+                ' of its status on the exchange site.'))
     sites = flow.Child(SiteSelection).ui(hidden=True)
 
+    _revision = flow.Parent()
+
     def get_buttons(self):
         self.source_site.exclude_choice.set(self.root().project().get_current_site().name())
         source_site_choices = self.source_site.choices()
@@ -849,6 +1069,13 @@ class Request(RequestAs):
         self.source_site.set(source_site_choices[0])
 
         return ['Request', 'Cancel']
+
+    def needs_dialog(self):
+        source_site_choices = self.source_site.choices()
+        if len(source_site_choices) == 1:
+            return False
+
+        return True
     
     def get_source_site_name(self):
         return self.source_site.get()
@@ -991,7 +1218,10 @@ class SiteMap(CustomEntityCollection):
         return Site
 
     def mapped_names(self, page_num=0, page_size=None):
-        return ["default"] + super(SiteMap, self).mapped_names(page_num, page_size)
+        names = super(SiteMap, self).mapped_names(page_num, page_size)
+        if "default" not in names:
+            names.insert(0, "default")
+        return names
     
     def columns(self):
         return ["Site"]
@@ -1067,8 +1297,26 @@ class Queue(flow.Object):
     job_list = flow.Child(JobQueue).ui(
         label="Jobs",
         expanded=True,
+        default_height=600,
         show_filter=True)
 
+    last_auto_sync = flow.Param().ui(editor='datetime', editable=False, hidden=True)
+    last_manual_sync = flow.Param().ui(editor='datetime', editable=False, hidden=True)
+
+    job_types_filter  = flow.SessionParam(None, PresetSessionValue)
+    job_status_filter = flow.SessionParam(None, PresetSessionValue)
+
+    def check_default_values(self):
+        self.job_types_filter.apply_preset()
+        self.job_status_filter.apply_preset()
+
+    def update_presets(self):
+        self.job_types_filter.update_preset()
+        self.job_status_filter.update_preset()
+
+    def _fill_ui(self,ui):
+        ui['custom_page'] = 'libreflow.baseflow.ui.job_queue.JobQueueWidget'
+
 
 class GotoQueue(flow.Action):
 
@@ -1096,46 +1344,6 @@ class GotoCurrentSiteQueue(flow.Action):
         return self.get_result(goto=current_site.queue.oid())
 
 
-class ClearQueueJobStatus(flow.values.ChoiceValue):
-
-    CHOICES = JobStatus.CHOICES + ["All"]
-
-
-class ClearQueueLoadType(flow.values.ChoiceValue):
-
-    CHOICES = LoadType.CHOICES + ["All"]
-
-
-class ClearQueueAction(flow.Action):
-
-    _site = flow.Parent()
-    status = flow.Param("PROCESSED", ClearQueueJobStatus)
-    type = flow.Param("All", ClearQueueLoadType)
-    emitted_before = flow.SessionParam((datetime.now() - timedelta(days=1)).timestamp()).ui(
-        editor='datetime'
-    )
-
-    def get_buttons(self):
-        self.message.set(
-            "<h2>Clear %s jobs</h2>" % self._site.name()
-        )
-        return ["Clear", "Cancel"]
-    
-    def run(self, button):
-        queue = self._site.get_queue()
-        status = self.status.get()
-        type = self.type.get()
-
-        if status == "All" and type == "All":
-            queue.clear()
-        else:
-            for job in queue.jobs(status=self.status.get()):
-                if job.date.get() < self.emitted_before.get():
-                    queue.remove(job.name())
-
-        queue.touch()
-
-
 class RemoveEnvironmentVariableAction(flow.Action):
 
     ICON = ('icons.gui', 'remove-symbol')
@@ -1189,9 +1397,9 @@ class AddEnvironmentVariableAction(flow.Action):
 
     _vars_collection = flow.Parent().ui(hidden=True)
     variable = flow.SessionParam("")
-    value_windows = flow.SessionParam('')
-    value_linux   = flow.SessionParam('')
-    value_darwin  = flow.SessionParam('')
+    value_windows = flow.SessionParam('').ui(editor='path')
+    value_linux   = flow.SessionParam('').ui(editor='path')
+    value_darwin  = flow.SessionParam('').ui(editor='path')
 
     def get_buttons(self):
         return ['Create', 'Cancel']
@@ -1229,7 +1437,7 @@ class AddEnvironmentVariableAction(flow.Action):
 
 class SiteEnvironment(flow.Map):
     
-    add_environment_variable = flow.Child(AddEnvironmentVariableAction)
+    add_environment_variable = flow.Child(AddEnvironmentVariableAction).ui(dialog_size=(600, 310))
 
     @classmethod
     def mapped_type(cls):
@@ -1270,10 +1478,10 @@ class WorkingSite(Site):
     is_kitsu_admin               = Property().ui(editor='bool')
     auto_upload_kitsu_playblasts = Property().ui(editor='bool')
 
-    root_folder         = flow.Computed(cached=True)
-    root_windows_folder = Property()
-    root_linux_folder   = Property()
-    root_darwin_folder  = Property()
+    root_folder         = flow.Computed(cached=True).ui(editor='path')
+    root_windows_folder = Property().ui(editor='path')
+    root_linux_folder   = Property().ui(editor='path')
+    root_darwin_folder  = Property().ui(editor='path')
     
     ordered_site_names  = flow.Computed(cached=True)
     custom_site_order   = Property().watched().ui(
@@ -1284,12 +1492,12 @@ class WorkingSite(Site):
     sync_up_max_connections = Property().ui(editor='int')
     pool_names = Property()
 
-    queue = flow.Child(Queue)
+    queue = flow.Child(Queue).ui(hidden=True)
     configuration = flow.Child(ConfigureWorkingSite)
     goto_queue = flow.Child(GotoQueue).ui(
         label="Show job queue"
     )
-    clear_queue = flow.Child(ClearQueueAction)
+    
     site_environment = flow.Child(SiteEnvironment)
 
     with flow.group('AfterEffects settings'):
@@ -1316,10 +1524,10 @@ class WorkingSite(Site):
             elif _os == "Darwin":
                 root_dir = self.root_darwin_folder.get()
             else:
-                print("ERROR: Unrecognized operative system ?")
+                self.root().session().log_error("[Working Site] Unrecognized operative system ?")
             
-            if not root_dir or not os.path.exists(root_dir):
-                print("WARNING: ROOT_DIR path DOES NOT EXISTS")
+            # if not root_dir or not os.path.exists(root_dir):
+            #     self.root().session().log_warning("[Working Site] ROOT_DIR path DOES NOT EXISTS")
 
             child_value.set(root_dir)
 
@@ -1355,6 +1563,15 @@ class WorkingSite(Site):
     def count_jobs(self, type=None, status=None):
         return self.get_queue().count(type=type, status=status)
 
+    def set_sync_date(self):
+        date = time.time()
+        
+        module_name = os.path.basename(sys.modules['__main__'].__file__)
+        if 'sync' in module_name:
+            self.queue.last_auto_sync.set(date)
+        else:
+            self.queue.last_manual_sync.set(date)
+
 
 class ClearSiteQueues(flow.Action):
 
@@ -1371,16 +1588,18 @@ class ClearSiteQueues(flow.Action):
             return
         
         for s in self._sites.mapped_items():
-            for j in s.get_jobs(status='PROCESSED'):
-                c = s.queue.job_list.get_entity_store().get_collection(
-                    s.queue.job_list.collection_name()
-                )
-                c.delete_many({
-                    'date': {'$lt': time.time() - self.emitted_since.get()},
-                    'status': 'PROCESSED'
-                })
+            # for j in s.get_jobs(status='PROCESSED'):
+            c = s.queue.job_list.get_entity_store().get_collection(
+                s.queue.job_list.collection_name()
+            )
+            c.delete_many({
+                'date': {'$lt': time.time() - self.emitted_since.get()},
+                'status': 'PROCESSED'
+            })
+            self.root().session().log_info(f'[Clear Site Queues] Clean {s.name()} job queue')
 
         self.root().project().synchronization.touch()
+        self.root().session().log_info(f'[Clear Site Queues] Cleaning complete')
 
 
 class WorkingSites(SiteMap):
@@ -1393,6 +1612,9 @@ class WorkingSites(SiteMap):
     def mapped_type(cls):
         return flow.injection.injectable(WorkingSite)
 
+    def columns(self):
+        return ["Site", "Last manual sync", "Last auto sync"]
+
     def _configure_child(self, child):
         if child.name() == "default":
             child.short_name.set("dft")
@@ -1409,6 +1631,23 @@ class WorkingSites(SiteMap):
         else:
             style['icon'] = ('icons.gui', 'home')
 
+    def _fill_row_cells(self, row, item):
+        super(WorkingSites, self)._fill_row_cells(row, item)
+
+        if item.queue.last_manual_sync.get() is None:
+            row["Last manual sync"] = ''
+        else:
+            row["Last manual sync"] = timeago.format(
+                datetime.fromtimestamp(item.queue.last_manual_sync.get()), datetime.now()
+            )
+
+        if item.queue.last_auto_sync.get() is None:
+            row["Last auto sync"] = ''
+        else:
+            row["Last auto sync"] = timeago.format(
+                datetime.fromtimestamp(item.queue.last_auto_sync.get()), datetime.now()
+            )
+
 
 class ConfigureExchangeSite(ConfigureSite):
 
@@ -1494,7 +1733,10 @@ class ExchangeSites(SiteMap):
         return flow.injection.injectable(ExchangeSite)
 
     def mapped_names(self, page_num=0, page_size=None):
-        return ["default_exchange"] + super(SiteMap, self).mapped_names(page_num, page_size)
+        names = super(SiteMap, self).mapped_names(page_num, page_size)
+        if "default_exchange" not in names:
+            names.insert(0, "default_exchange")
+        return names
 
     def _configure_child(self, child):
         if child.name() == "default_exchange":
