@@ -1,4 +1,4 @@
-from PySide2.QtCharts import QtCharts
+from PySide6 import QtCharts
 
 
 class HistoryGraph(QtCharts.QChart):
