@@ -30,7 +30,7 @@ class FileActionsButton(QtWidgets.QToolButton):
 
         # Add actions
         self.menu = QtWidgets.QMenu('File actions')
-        self.setPopupMode(QtWidgets.QToolButton.InstantPopup)
+        self.setPopupMode(QtWidgets.QToolButton.ToolButtonPopupMode.InstantPopup)
         self.setArrowType(QtCore.Qt.NoArrow)
         self.setMenu(self.menu)
     
@@ -39,15 +39,15 @@ class FileActionsButton(QtWidgets.QToolButton):
         data.update_actions()
 
         has_actions = self.action_manager.update_oid_menu(
-            data.oid, self.menu, with_submenus=True
+            data.file_oid, self.menu, with_submenus=True
         )
 
-        if data.is_ref:
+        if data.ref_oid is not None:
             if has_actions:
                 self.menu.addSeparator()
             
             a = self.menu.addAction(
-                'Unlink', lambda oid=data.oid, file_type=self.file_type: self.controller.remove_ref(oid, file_type)
+                'Unlink', lambda oid=data.ref_oid, file_type=self.file_type: self.controller.remove_ref(oid, file_type)
             )
             a.setIcon(resources.get_icon(('icons.gui', 'ref-broken')))
 
@@ -68,23 +68,32 @@ class FileListItemWidget(QtWidgets.QWidget):
         self.file_type = file_type
         self.row = row
         self.flow_page = flow_page
+        self.rect_text_width = 0
+        self.metrics_text_width = 0
+        
         self.build()
-    
+        self.installEventFilter(self)
+
     def build(self):
+        self.data = self.controller.file_data(self.file_type, self.row)
+
+        self.buttons_widget = QtWidgets.QWidget()
+        self.buttons_lo = QtWidgets.QHBoxLayout(self.buttons_widget)
+        self.buttons_lo.setContentsMargins(0,0,0,0)
+        self.buttons_lo.setSpacing(0)
+       
         self.button_secondary = FileActionsButton(self.flow_page, self.file_type, self.row, self.action_manager, self.controller)
-        self.button_secondary.setFixedWidth(30)
+        self.button_secondary.setFixedSize(25, 25)
         self.button_secondary.setSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Expanding)
 
         self.buttons_main = []
-        data = self.controller.file_data(self.file_type, self.row)
-        data.update_actions()
 
-        for fa in data.main_actions:
+        for fa in self.data.main_actions:
             b = QtWidgets.QPushButton('')
             b.setIcon(resources.get_icon(fa.icon))
             b.setToolTip(fa.label)
-            b.setFixedWidth(30)
-            b.setStyleSheet('background-color: rgba(255, 255, 255, 0);')
+            b.setFixedSize(self.button_secondary.size())
+            b.setStyleSheet('QPushButton { background-color: rgba(255, 255, 255, 0); }')
             b.clicked.connect(lambda checked=False, a=fa: self._on_action_menu_triggered(a))
             self.buttons_main.append(b)
         
@@ -92,13 +101,42 @@ class FileListItemWidget(QtWidgets.QWidget):
         hlo.addStretch(1)
 
         for b in self.buttons_main:
-            hlo.addWidget(b)
+            self.buttons_lo.addWidget(b)
         
+        hlo.addWidget(self.buttons_widget)
         hlo.addWidget(self.button_secondary)
         hlo.setSpacing(0)
-        hlo.setMargin(0)
+        hlo.setContentsMargins(4, 0, 4, 0)
         self.setLayout(hlo)
 
+    def get_buttons_count(self):
+        count = 0
+        for i in reversed(range(self.buttons_lo.count())):
+            widget = self.buttons_lo.itemAt(i).widget()
+            if widget.isVisible():
+                count += 1
+        
+        return count
+    
+    # Make main actions responsive
+    def eventFilter(self, object, event):
+        if event.type() == QtCore.QEvent.Resize or event.type() == QtCore.QEvent.Paint:
+            if self.rect_text_width < self.metrics_text_width:
+                for btn in self.buttons_main:
+                    if btn.isVisible():
+                        btn.hide()
+                        break
+
+            elif (self.rect_text_width - self.metrics_text_width) >= self.button_secondary.width():
+                for btn in reversed(self.buttons_main):
+                    if not btn.isVisible():
+                        btn.show()
+                        break
+
+            return True
+
+        return False
+
     def _on_action_menu_triggered(self, action):
         self.flow_page.show_action_dialog(action.oid)
 
@@ -109,6 +147,7 @@ class FileList(QtWidgets.QTableView):
     """
     def __init__(self, task_widget, file_type):
         super(FileList, self).__init__()
+        self.task_widget = task_widget
         self.controller = task_widget.controller
         self.session = task_widget.controller.session
         self.file_type = file_type
@@ -118,10 +157,12 @@ class FileList(QtWidgets.QTableView):
         self.setItemDelegate(QFileListDelegate())
         self.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
         self.verticalHeader().hide()
+        self.verticalHeader().setDefaultSectionSize(35)
+        self.horizontalHeader().setDefaultAlignment(QtCore.Qt.AlignVCenter)
         self.horizontalHeader().setStretchLastSection(True)
-        self.setVerticalScrollMode(QtWidgets.QAbstractItemView.ScrollPerPixel)
+        self.setVerticalScrollMode(QtWidgets.QAbstractItemView.ScrollMode.ScrollPerPixel)
         self.setDragEnabled(True)
-        self.setDragDropMode(QtWidgets.QAbstractItemView.InternalMove)
+        self.setDragDropMode(QtWidgets.QAbstractItemView.DragDropMode.InternalMove)
 
         self.action_manager = ObjectActionMenuManager(
             task_widget.session, task_widget.page.show_action_dialog, 'Flow.map'
@@ -132,6 +173,7 @@ class FileList(QtWidgets.QTableView):
 
         self.customContextMenuRequested.connect(self._on_context_menu_requested)
         self.doubleClicked.connect(self._on_item_double_clicked)
+        self.installEventFilter(self)
     
     def update(self):
         for row in range(self.model.rowCount()):
@@ -139,11 +181,22 @@ class FileList(QtWidgets.QTableView):
                 self.model.index(row, 0),
                 FileListItemWidget(self.controller.task_widget.page, self.file_type, row, self.action_manager, self.controller)
             )
-    
+
+    def eventFilter(self, source, event):
+        if event.type() == QtCore.QEvent.Hide:
+            # Reset selection when the widget is hidden
+            QtWidgets.QApplication.instance().selectChanged.emit(None)
+            return True
+        
+        return super().eventFilter(source, event)
+
     def selectionChanged(self, selected, deselected):
         if selected.indexes():
             index = selected.indexes()[0]
             self.controller.update_selected(self.file_type, index.row())
+        else:
+            QtWidgets.QApplication.instance().selectChanged.emit(None)
+            self.task_widget.setFocus()
     
     def mousePressEvent(self, event):
         super(FileList, self).mousePressEvent(event)
@@ -156,6 +209,8 @@ class FileList(QtWidgets.QTableView):
             if event.button() == QtCore.Qt.LeftButton:
                 if not self.indexAt(event.pos()).isValid():
                     self.controller.clear_selected()
+                else:
+                    QtWidgets.QApplication.instance().selectChanged.emit(self.controller.selected_file().file_oid)
 
     def dragEnterEvent(self, event):
         if not event.mouseButtons() & QtCore.Qt.LeftButton:
@@ -182,7 +237,7 @@ class FileList(QtWidgets.QTableView):
             links = []
             for url in event.mimeData().urls():
                 links.append(str(url.toLocalFile()))
-            print(links)
+            
             return self.controller.handle_dropped_files(links)
 
         md = {}
@@ -211,19 +266,19 @@ class FileList(QtWidgets.QTableView):
         
         data = self.controller.file_data(self.file_type, index.row())
         has_actions = self.action_manager.update_oid_menu(
-            data.oid, self.action_menu, with_submenus=True
+            data.file_oid, self.action_menu, with_submenus=True
         )
 
-        if data.is_ref:
+        if data.ref_oid is not None:
             if has_actions:
                 self.action_menu.addSeparator()
             
             a = self.action_menu.addAction(
-                'Unlink', lambda oid=data.oid, file_type=self.file_type: self.controller.remove_ref(oid, file_type)
+                'Unlink', lambda oid=data.ref_oid, file_type=self.file_type: self.controller.remove_ref(oid, file_type)
             )
             a.setIcon(resources.get_icon(('icons.gui', 'ref-broken')))
 
-        if has_actions or data.is_ref:
+        if has_actions or data.ref_oid is not None:
             self.action_menu.exec_(self.viewport().mapToGlobal(pos))
     
     def _on_item_double_clicked(self, index):
@@ -259,7 +314,7 @@ class FileListActionsWidget(QtWidgets.QWidget):
             hlo.addWidget(b)
         
         hlo.setSpacing(0)
-        hlo.setMargin(0)
+        hlo.setContentsMargins(0, 0, 0, 0)
         self.setLayout(hlo)
 
     def _on_action_menu_triggered(self, action):
@@ -288,7 +343,7 @@ class FileListsWidget(QtWidgets.QWidget):
         glo.addWidget(self.file_lists['Outputs'], 0, 2)
         glo.addWidget(self.file_buttons, 1, 0, 1, 3)
         glo.setSpacing(2)
-        glo.setMargin(0)
+        glo.setContentsMargins(0, 0, 0, 0)
         self.setLayout(glo)
     
     def clear_list_selection(self, file_type):
