@@ -83,7 +83,7 @@ class UserProfile(flow.Object):
     ICON = ('icons.gui', 'user')
 
     current_user_id = flow.Computed(cached=True)
-    my_bookmarks = flow.Child(MyBookmarks).ui(expanded=True)
+    my_bookmarks = flow.Child(MyBookmarks)
     preferences = flow.Child(GotoPreferences)
 
     def compute_child_value(self, child_value):
@@ -116,10 +116,6 @@ class UserProfile(flow.Object):
                 ' undefined.</font>'
             )
 
-    def _fill_ui(self, ui):
-        if self.root().project().show_login_page():
-            ui["custom_page"] = "libreflow.baseflow.LoginPageWidget"
-
 
 
 
@@ -435,7 +431,7 @@ class User(CustomEntity):
 
     login  = Property()
     code   = Property()
-    status = Property(UserStatus).ui(editable=False)
+    status = Property(UserStatus)
 
     bookmarks = flow.Child(UserBookmarks)
     preferences = flow.Child(UserPreferences)
@@ -471,7 +467,7 @@ class Users(CustomEntityCollection):
     def add_user(self, name, login, status='User'):
         user = self.add(name)
         user.status.set(status)
-        user.login.set(login)
+        user.login.set([login])
 
         return user
     
@@ -486,7 +482,7 @@ class Users(CustomEntityCollection):
             self.get_entity_store()
             .get_collection(self.collection_name())
             .find_one(
-                {'login': login},
+                {'login': {'$regex': login}},
                 {'name': 1}
             )
         )
@@ -505,6 +501,8 @@ class Users(CustomEntityCollection):
 
 class AddEnvVarAction(flow.Action):
 
+    ICON = ("icons.gui", "plus-sign-in-a-black-circle")
+
     _env = flow.Parent()
 
     var_name = flow.SessionParam("").ui(label="Name")
@@ -547,14 +545,19 @@ class AddEnvVarAction(flow.Action):
         self.var_value.revert_to_default()
 
 
-class ChangeEnvVarValueAction(flow.Action):
+class ChangeEnvVarAction(flow.Action):
+
+    ICON = ('icons.libreflow', 'edit-blank')
 
     _var = flow.Parent()
     _env = flow.Parent(2)
 
+    var_name = flow.Param("").ui(label="Name")
     var_value = flow.Param("").ui(label="Value")
 
     def get_buttons(self):
+        self.var_name.set(self._var.name())
+        self.var_value.set(self._var.get())
         return ["Confirm", "Cancel"]
 
     def run(self, button):
@@ -569,19 +572,55 @@ class ChangeEnvVarValueAction(flow.Action):
             return
 
         env = json.load(f)
-        env[self._var.name()] = self.var_value.get()
+        # Delete old variable if name is changed
+        if self._var.name() != self.var_name.get():
+            env.pop(self._var.name())
+            env[self.var_name.get()] = self.var_value.get()
+        else:
+            env[self._var.name()] = self.var_value.get()
 
         f = open(env_path, "w")
         json.dump(env, f, indent=4, sort_keys=True)
         f.close()
 
-        self._var.set(self.var_value.get())
-        self._var.update()
+        self._env.touch()
+
+
+class RemoveEnvVarAction(flow.Action):
+
+    ICON = ('icons.gui', 'remove-symbol')
+
+    _var = flow.Parent()
+    _env = flow.Parent(2)
+
+    def needs_dialog(self):
+        return False
+
+    def run(self, button):
+        if button == "Cancel":
+            return
+
+        env_path = self._env.file_path()
+
+        try:
+            f = open(env_path, "r")
+        except IOError:
+            return
+
+        env = json.load(f)
+        env.pop(self._var.name())
+
+        f = open(env_path, "w")
+        json.dump(env, f, indent=4, sort_keys=True)
+        f.close()
+
+        self._env.touch()
 
 
 class EnvVar(flow.values.SessionValue):
 
-    change_value = flow.Child(ChangeEnvVarValueAction)
+    change_value = flow.Child(ChangeEnvVarAction).ui(label="Edit")
+    remove_value = flow.Child(RemoveEnvVarAction).ui(label="Remove")
 
     def update(self):
         pass
@@ -610,6 +649,11 @@ class UserEnvironment(flow.DynamicMap):
 
         return env.keys()
 
+    def summary(self):
+        message = f'<font color="#EFDD5B">WARNING: Only the current libreflow and local user can use these variables.</font>'
+
+        return message
+
     def file_path(self):
         return "%s/env.json" % self.root().project().user_settings_folder()
 
