@@ -2,6 +2,7 @@ from __future__ import print_function
 
 import six
 import os
+import logging
 from kabaret import flow
 from pprint import pprint
 from kabaret.app.ui.gui.widgets.flow.flow_view import (
@@ -24,7 +25,7 @@ class ProjectThumbnailWidget(QtWidgets.QAbstractButton):
             pixmap.loadFromData(ba, image.split(';')[0].split('/')[1])
             self.thumbnail = pixmap
         elif image_path:
-            print("loading thumbnail from path")
+            logging.getLogger('kabaret').log(logging.DEBUG, "loading thumbnail from path")
             self.thumbnail = QtGui.QPixmap(image_path)
         else:
             self.setThumnail = False
@@ -130,6 +131,7 @@ class ProjectHomePageWidget(CustomPageWidget):
                 continue
 
             project_list.addWidget(ProjectWidget(None, name, self))
+        project_list.addStretch()
         self.setLayout(project_list)
 
         # Go directly to Wizard if no project has been created.
