@@ -144,6 +144,8 @@ def main(argv):
         db,
         password,
         debug,
+        read_replica_host,
+        read_replica_port,
         clean_time,
         nb_days_before,
         limit_time,
@@ -163,7 +165,7 @@ def main(argv):
         verbose=verbose,
         force_delete=force_delete
     )
-    session.cmds.Cluster.connect(host, port, cluster_name, db, password)
+    session.cmds.Cluster.connect(host, port, cluster_name, db, password, read_replica_host, read_replica_port)
 
     session.start()
 
