@@ -13,7 +13,7 @@ def process_remaining_args(args):
         '-r', '--root_dir', dest='root_dir'
     )
     parser.add_argument(
-        '-s', '--site', default='LFS', dest='site'
+        '-s', '--site', default=os.getenv('LIBREFLOW_SITE', 'lfs'), dest='site'
     )
     parser.add_argument(
         '--blender_exec_path', dest='blender_exec_path'
@@ -32,14 +32,14 @@ def process_remaining_args(args):
 
 if __name__ == '__main__':
     argv = sys.argv[1:]  # get ride of first args wich is script filename
-    session_name, host, port, cluster_name, db, password, debug, remaining_args \
+    session_name, host, port, cluster_name, db, password, debug, read_replica_host, read_replica_port, remaining_args \
         = JobsNodeSession.parse_command_line_args(argv)
     pools = process_remaining_args(remaining_args)
     if not pools:
         print('\n!!!\nUsage:', sys.argv[0], 'pool_name pool_name ...\n\n')
     else:
         node = JobsNodeSession(pools, session_name=session_name, debug=debug)
-        node.cmds.Cluster.connect(host, port, cluster_name, db, password)
+        node.cmds.Cluster.connect(host, port, cluster_name, db, password, read_replica_host, read_replica_port)
 
         node.wait_for_jobs()
         print('Jobs Node closed.')
