@@ -25,6 +25,7 @@ import json
 import argparse
 import fileseq
 import textwrap
+import shutil
 from tempfile import mkdtemp
 from math import inf
 from concurrent.futures import ThreadPoolExecutor
@@ -32,19 +33,6 @@ from concurrent.futures import ThreadPoolExecutor
 
 __all__ = ['default_template', 'SequenceMarker']
 
-# Paths of installed binaries, depending on environment
-# It would be better to let the OS handle the path, but for now let's not
-bins = {'Linux':
-        {'ffmpeg': 'ffmpeg',
-         'convert': 'convert'},
-        'Windows':
-        {'ffmpeg': 'ffmpeg',
-         'convert': 'convert'},
-        'Darwin':
-        {'ffmpeg': 'ffmpeg',
-         'convert': 'convert'}
-}
-
 
 default_template = {
     "settings": {
@@ -156,9 +144,10 @@ def frames_to_timecode(frames, fps=24):
 
 
 class SequenceMarker():
-    def __init__(self, image_filepath, data, template=default_template):
+    def __init__(self, image_filepath, data, convert_bin, template=default_template):
         self.data = data
         self.template = template
+        self.convert_bin = convert_bin
         self.create_temp_dir()
 
         self.file_sequence = fileseq.findSequenceOnDisk(image_filepath)
@@ -242,8 +231,7 @@ class SequenceMarker():
         specified image'''
         print("Marking image %s..." % image_data['frame_number'])
 
-        convert_bin = bins[platform.system()]['convert']
-        convert_args = [convert_bin]
+        convert_args = [self.convert_bin]
         convert_args += ['%s' % path]
 
         settings = self.template['settings']
@@ -272,7 +260,7 @@ class SequenceMarker():
                                 os.path.join(os.path.dirname(__file__),
                                 '../resources/mark_sequence/fonts/LiberationMono-Regular.ttf'
                                 )
-                             ),
+                             ).replace('\\', '/'),
                              '-pointsize', str(settings['font_size'])])
 
         # Add annotations for each field, for outside stroke
@@ -314,7 +302,7 @@ class SequenceMarker():
 
     def render_video(self, img_sources, destination, audio_file=None, frame_rate=25):
         print("Generating video...")
-        ffmpeg_bin = bins[platform.system()]['ffmpeg']
+        ffmpeg_bin = 'ffmpeg'
         ffmpeg_args = [ffmpeg_bin, '-y', '-loglevel', 'error']
         ffmpeg_args.extend(['-r', str(frame_rate)])
         ffmpeg_args.extend(['-i', img_sources])
@@ -395,6 +383,21 @@ if __name__ == "__main__":
 
     args = parser.parse_known_args()[0]
 
+    #Check for convert in PATH
+    magick_command = 'magick'
+
+    if not shutil.which('magick'):
+        magick_command = 'convert'
+        print('[Mark Image Sequence]: "magick" command not found in PATH, fallback to "convert"')
+        convert_path = shutil.which('convert')
+        if convert_path is None or 'ImageMagick' not in convert_path:
+            print('[Mark Image Sequence]: "convert" command not found in PATH, marking interrupted')
+            exit()
+
+    if not shutil.which('ffmpeg'):
+        print('[Mark Sequence]: "ffmpeg" command not found in PATH, marking interrupted')
+        exit()
+
     # Load in template from supplied json file. If none given, use default one.
     if args.template is None:
         template = default_template
@@ -416,7 +419,7 @@ if __name__ == "__main__":
 
     args = parser.parse_args()
 
-    sequence_marker = SequenceMarker(os.path.abspath(args.image_sequence), vars(args), template=template)
+    sequence_marker = SequenceMarker(os.path.abspath(args.image_sequence), vars(args), convert_bin=magick_command, template=template)
 
     # Get resolution from first image
     res = subprocess.check_output(['identify', '-format', '%wx%h',
