@@ -1,9 +1,7 @@
-from qtpy import QtCore
 
 from kabaret.app.ui import gui
 from kabaret.app.session import KabaretSession
-from kabaret.script_view import ScriptView
-from kabaret.subprocess_manager import SubprocessManager, LauncherToolBar
+from kabaret.subprocess_manager.actor import SubprocessManager
 
 from libreflow.utils.kabaret.jobs.jobs_node import JobsNodeSession
 
@@ -21,14 +19,6 @@ class BaseGUISession(gui.KabaretStandaloneGUISession):
         subprocess_manager = SubprocessManager(self)
 
 
-class DebugGUISession(BaseGUISession):
-    def register_view_types(self):
-        super(DebugGUISession, self).register_view_types()
-
-        type_name = self.register_view_type(ScriptView)
-        self.add_view(type_name, hidden=True, area=QtCore.Qt.RightDockWidgetArea)
-
-
 class BaseCLISession(KabaretSession):
     def _create_actors(self):
         super(BaseCLISession, self)._create_actors()
