@@ -132,23 +132,33 @@ class CustomEntityCollection(EntityCollection):
         self.get_entity_store().get_collection(self.collection_name()).update_one(
             {"name": self._get_mapped_item_name(entity_name)},
             {"$set": {property_name: value}},
+            upsert=True
         )
 
     def get_property(self, entity_name, property_name):
         self.root().session().log_debug(f'===========> {self._get_mapped_item_name(entity_name)} {property_name}')
-        value = (
-            self.get_entity_store()
-            .get_collection(self.collection_name())
-            .find_one(
-                {"name": self._get_mapped_item_name(entity_name)},
-                {property_name: 1},
-            )
-        )
+        name = self._get_mapped_item_name(entity_name)
+        
+        if (
+            self._document_cache is None
+            or self._document_cache_birth < time.time() - self._document_cache_ttl
+            or entity_name not in self._document_cache
+        ):
+            value = (
+                self.get_entity_store()
+                .get_collection(self.collection_name())
+                .find_one(
+                    {"name": name},
+                    {property_name: 1},
+                )
+             )
+        else:
+            value = self._document_cache[name]
+        
         try:
             return value[property_name]
         except (KeyError, TypeError):
-            default = getattr(self._get_mapped_item_type(entity_name), property_name).get_default_value()
-            return default
+            return getattr(self._get_mapped_item_type(entity_name), property_name).get_default_value()
     
     def get_properties(self, entity_name, *property_names):
         '''
@@ -156,23 +166,32 @@ class CustomEntityCollection(EntityCollection):
 
         This method assumes that all the requested properties exist in this entity.
         '''
-        doc = (
-            self.get_entity_store()
-            .get_collection(self.collection_name())
-            .find_one(
-                {"name": self._get_mapped_item_name(entity_name)},
-                {
-                    property_name: True
-                    for property_name in property_names
-                },
+        name = self._get_mapped_item_name(entity_name)
+
+        if (
+            self._document_cache is None
+            or self._document_cache_birth < time.time() - self._document_cache_ttl
+            or entity_name not in self._document_cache
+        ):
+            doc = (
+                self.get_entity_store()
+                .get_collection(self.collection_name())
+                .find_one(
+                    {"name": name},
+                    {
+                        property_name: True
+                        for property_name in property_names
+                    },
+                )
             )
-        )
+        else:
+            doc = self._document_cache[name]
         values = {}
 
         for property_name in property_names:
             try:
                 values[property_name] = doc[property_name]
-            except KeyError:
+            except (KeyError, TypeError):
                 values[property_name] = getattr(self.mapped_type(), property_name).get_default_value()
 
         return values
