@@ -322,7 +322,10 @@ class Job(object):
         if job.get_in_progress() and not forced:
             return False
 
-        job.remove_from_pool_queue()
+        # FIXME: pool may be undefined because of an error
+        # during job creation using a Redis replica
+        if job.get_pool() is not None:
+            job.remove_from_pool_queue()
 
         actor._db.srem(actor.jobids_set_name, jid)
         nb = actor._db.delete(actor.job_hashes_namespace+'/'+jid)
