@@ -11,19 +11,10 @@ from __future__ import print_function
 import sys
 import os
 import traceback
-import sentry_sdk
 
 from kabaret.app.session import KabaretSession
-import libreflow.utils.kabaret as kutils
+from kabaret.subprocess_manager import SubprocessManager
 from libreflow.utils.kabaret.jobs import jobs_actor
-from libreflow.baseflow.runners import (
-    Blender,
-    AfterEffectsRender,
-    SessionWorker,
-    MarkSequenceRunner,
-)
-
-from libreflow.resources import file_templates
 
 
 def try_initialize_maya():
@@ -54,25 +45,12 @@ class JobsWorkerSession(KabaretSession):
     def _create_actors(self):
         super(JobsWorkerSession, self)._create_actors()
         self.jobs_actor = jobs_actor.Jobs(self)
-        
-        subprocess_manager = kutils.subprocess_manager.SubprocessManager(self)
-
-        factory = subprocess_manager.create_new_factory('JobsWorker Tools')
-        factory.ensure_runner_type(Blender)
-        factory.ensure_runner_type(AfterEffectsRender)
-        factory.ensure_runner_type(MarkSequenceRunner)
-        factory.ensure_runner_type(SessionWorker)
-
-        subprocess_manager.ensure_factory(factory)
+        subprocess_manager = SubprocessManager(self)
 
     def execute_job(self):
         self.jobs_actor.worker_execute_job()
 
 
 if __name__ == '__main__':
-    if 'SENTRY_DSN' in os.environ:
-        print('SENTRY_DSN found!')
-        sentry_sdk.init(os.environ['SENTRY_DSN'], traces_sample_rate=1.0)
-    
     try_initialize_maya()
     JobsWorkerSession()
\ No newline at end of file
