@@ -190,9 +190,9 @@ class JobList(QtWidgets.QTreeWidget):
         column_names, columns_visible = self.header().get_columns()
         self.setHeaderLabels(column_names)
 
-        self.header().setSectionResizeMode(QtWidgets.QHeaderView.ResizeToContents)
-        self.header().setSectionResizeMode(0, QtWidgets.QHeaderView.Interactive)
-        self.header().setSectionResizeMode(3, QtWidgets.QHeaderView.Stretch)
+        self.header().setSectionResizeMode(QtWidgets.QHeaderView.ResizeMode.ResizeToContents)
+        self.header().setSectionResizeMode(0, QtWidgets.QHeaderView.ResizeMode.Interactive)
+        self.header().setSectionResizeMode(3, QtWidgets.QHeaderView.ResizeMode.Stretch)
         
         for col_i, col_visible in enumerate(columns_visible):
             self.setColumnHidden(col_i, not col_visible)
@@ -361,4 +361,20 @@ class JobsView(DockedView):
             self._job_list.create_job(jid)
         elif event == 'job_deleted':
             jid = data['jid']
-            self._job_list.delete_job(jid)
\ No newline at end of file
+            self._job_list.delete_job(jid)
+
+        if event == "focus_changed":
+            # Update dock title bar background color depending on the active view status
+            view_id = data["view_id"]
+
+            titlebar = self.dock_widget().titleBarWidget()
+            if not titlebar:
+                return
+
+            dock_background = titlebar.get_container()
+            dock_background.setProperty(
+                "current", True if view_id == self.view_id() else False
+            )
+
+            dock_background.style().polish(dock_background)
+            dock_background.update()
\ No newline at end of file
