@@ -1,6 +1,7 @@
 import six
 import qtpy
 import functools
+import logging
 
 from kabaret.app.ui.gui.widgets.flow.flow_view import QtWidgets, QtGui, QtCore
 from kabaret.app.ui.gui.widgets.flow_layout import FlowLayout
@@ -13,17 +14,12 @@ class Separator(QtWidgets.QToolButton):
         super(Separator, self).__init__(related_navigation_button)
         self.setProperty("class", "nav_separator")
         self.related_navigation_button = related_navigation_button
-        self.setIcon(resources.get_icon(
-            ('icons.flow', 'collapsed'),
-            disabled_ref=('icons.flow', 'collapsed')
-        ))
+        self.setText('>')
     
     def mousePressEvent(self, e):
         self.related_navigation_button._show_navigables_menu(self.related_navigation_button.nav_widget.current_oid())
         super(Separator, self).mousePressEvent(e)
 
-    def sizeHint(self):
-        return QtCore.QSize(18, 24)
 
 class NavigationButton(QtWidgets.QToolButton):
 
@@ -52,7 +48,7 @@ class NavigationButton(QtWidgets.QToolButton):
 
     def _goto_oid(self, oid):
         self.nav_widget._goto(oid)
-
+    
     def _goto(self, b=None):
         self.nav_widget._goto(self.oid)
 
@@ -74,10 +70,6 @@ class NavigationButton(QtWidgets.QToolButton):
             m.clear()
             m.addAction('ERROR: ' + str(err))
             raise
-        print (navigatable_entries)
-        print(session.cmds.Flow.get_navigable_oids(
-                self.oid, "/MyHomeRoot"
-            ))
         m.clear()
         root = m
         for i, entry in enumerate(navigatable_entries, 1):
@@ -129,7 +121,7 @@ class NavigationButton(QtWidgets.QToolButton):
 
         # below makes the pixmap half transparent
         painter = QtGui.QPainter(pixmap)
-        painter.setCompositionMode(painter.CompositionMode_DestinationIn)
+        painter.setCompositionMode(painter.CompositionMode.CompositionMode_DestinationIn)
         painter.fillRect(pixmap.rect(), QtGui.QColor(0, 0, 0, 127))
         painter.end()
 
@@ -159,7 +151,7 @@ class NavigationOIDControls(QtWidgets.QWidget):
 
         self.nav_oid_bar = parent
         self.bt_font = self.font()
-        self.bt_font.setPointSize(self.bt_font.pointSize() * 1.3)
+        self.bt_font.setPointSize(int(self.bt_font.pointSize()))
 
         self._flow_lo = FlowLayout()
         self.setLayout(self._flow_lo)
@@ -195,71 +187,130 @@ class NavigationOIDControls(QtWidgets.QWidget):
         return self._navigator.current_oid()
 
 
+class SearchEntriesStringListModel(QtCore.QStringListModel):
+
+    def __init__(self, line_edit):
+        super(SearchEntriesStringListModel, self).__init__()
+        self.line_edit = line_edit
+
+    def data(self, index, role):
+        if self.line_edit.search_enabled():
+            res = self.line_edit.get_search_result(index.row())
+            if role == QtCore.Qt.EditRole:
+                return res['goto_oid']
+            elif role == QtCore.Qt.DisplayRole:
+                return res['label']
+        else:
+            if role == QtCore.Qt.DisplayRole or role == QtCore.Qt.EditRole:
+                return self.line_edit.get_completion(index.row())
+
+
+class CustomListView(QtWidgets.QListView):
+    '''
+    List view used by the OID field's completer to validate
+    instantly a clicked result.
+    '''
+    def __init__(self, oid_field):
+        super(CustomListView, self).__init__()
+        self._oid_field = oid_field
+        
+    def mouseReleaseEvent(self, e):
+        self._oid_field.accept()
+        super(CustomListView, self).mouseReleaseEvent(e)
+        self.hide()
+
+
 class NavigationOIDField(QtWidgets.QLineEdit):
     def __init__(self, parent, navigator):
         super(NavigationOIDField, self).__init__(parent)
         self._navigator = navigator
         self.session = navigator.session
         self.nav_oid_bar = parent
+        
         field_font = self.font()
-        field_font.setPointSize(field_font.pointSize() * 1.3)
-
+        field_font.setPointSize(int(field_font.pointSize()))
         self.setFont(field_font)
 
-        self.textEdited.connect(self.update_completion)
-        self.completion_current_oid = ""  
+        self._completions = None
+        self._search_results = None
+        self.completer_view = CustomListView(self)
         self.completer = QtWidgets.QCompleter([], self)
         self.completer.setCaseSensitivity(QtCore.Qt.CaseInsensitive)
+        self.completer.setModel(SearchEntriesStringListModel(self))
+        self.completer.setCompletionMode(QtWidgets.QCompleter.UnfilteredPopupCompletion)
+        self.completer.setPopup(self.completer_view)
         self.setCompleter(self.completer)
 
+        self.textEdited.connect(self.on_text_edited)
+
+    def on_text_edited(self, text):
+        # self.completer.model().beginResetModel()
+
+        if text.startswith('/'):
+            self.set_search_enabled(False)
+            self._completions = self.update_completion(text)
+            self.completer.model().setStringList(self._completions)
+        else:
+            self.set_search_enabled(True)
+            self._search_results = self.update_search_results(text)
+            self.completer.model().setStringList([r['goto_oid'] for r in self._search_results])
+        
+        # self.completer.model().endResetModel()
+    
+    def get_search_result(self, index):
+        return self._search_results[index]
+    
+    def get_completion(self, index):
+        return self._completions[index]
+
     def get_completion_current_oid(self, text):
-        if not text:
-            return "/"
         if text.endswith("/"):
             return text
-        if text.startswith("/"):
-            return text.rpartition("/")[0] + "/"
         else:
-            return "/"
+            return text.rpartition("/")[0] + "/"
 
     def update_completion(self, text):
         current_oid = self.get_completion_current_oid(text)
-        if current_oid == self.completion_current_oid:
-            return
-        self.completion_current_oid = current_oid
+        completion_oid_list = []
+
+        def exists(oid):
+            ret = False
+            try:
+                ret = self.session.cmds.Flow.exists(oid)
+            except:
+                pass
+            return ret
 
         if not current_oid or current_oid == "/":
-            _target_oid = True
-        else:
-            try:
-                _target_oid = self.session.cmds.Flow.exists(current_oid)
-            except:
-                _target_oid = False
-                # TODO : RAISE ERROR ?
-                print("INVALID PROJECT OID ? : ", current_oid)
-                self.setProperty('error', True)
-
-        if _target_oid:
-            if not current_oid or current_oid == "/":
-                # Get list of project
-                projects = self.session.get_actor("Flow").get_projects_info()
-                completion_oid_list = list(map(lambda p: "/" + p[0], projects))
-                # Add the Home Oid to the autocompletion list
-                completion_oid_list.append(self.session.get_actor("Flow").home_root().Home.oid())
-            else:
-                # get list of children of current oid
-                navigatable_entries =  self.session.cmds.Flow.get_navigable_oids(
-                    current_oid.rpartition("/")[0], current_oid.rpartition("/")[0]
-                )
-                completion_oid_list = map(lambda navigatable_entry: navigatable_entry[1] ,navigatable_entries[navigatable_entries.index(None)+1:])
-            self.completer.model().setStringList(completion_oid_list)
-            
-
+            # Get list of project
+            projects = self.session.get_actor("Flow").get_projects_info()
+            completion_oid_list = list(map(lambda p: "/" + p[0], projects))
+            # Add the Home Oid to the autocompletion list
+            completion_oid_list.append(self.session.get_actor("Flow").home_root().Home.oid())
+        elif exists(current_oid):
+            # get list of children of current oid
+            navigatable_entries =  self.session.cmds.Flow.get_navigable_oids(
+                current_oid.rpartition("/")[0], current_oid.rpartition("/")[0]
+            )
+            completion_oid_list = list(map(lambda navigatable_entry: navigatable_entry[1] ,navigatable_entries[navigatable_entries.index(None)+1:]))
         else:
             # TODO : RAISE ERROR ?
-            print("INVALID OID : ", current_oid)
+            logging.getLogger('kabaret').log(logging.DEBUG, f"INVALID OID : {current_oid}")
             self.setProperty('error', True)
 
+        return completion_oid_list
+    
+    def update_search_results(self, text): 
+        results,count = self.session.cmds.Search.query_project_index(
+            self.nav_oid_bar.selected_project(), text, exclude_types=["TrackedFile", "TrackedFolder"])
+        return results
+    
+    def search_enabled(self):
+        return self.nav_oid_bar.search_enabled()
+    
+    def set_search_enabled(self, b):
+        self.nav_oid_bar.set_search_enabled(b)
+
     def reset(self):
         self.setText(self._navigator.current_oid())
         self.setProperty('edited', False)
@@ -274,33 +325,54 @@ class NavigationOIDField(QtWidgets.QLineEdit):
     def accept(self):
         oid = self.text().strip()
         # HACK to remove a final '/' as user might forget it in manual editing of OIDs
-        if oid[-1] == "/":
-            oid = oid[:-1]
+        if len(oid) > 0:
+            if oid[-1] == "/":
+                oid = oid[:-1]
         try:
             exists = self._navigator.session.cmds.Flow.exists(oid)
         except:
             exists = False
         if not exists:
-            # TODO : RAISE ERROR ?
-            self.session.logger.error("INVALID OID ? : {}".format(oid))
-            self.setProperty("error", True)
+
+            # redirection vers la page de recherche
+
+            in_new_view = (
+            QtWidgets.QApplication.keyboardModifiers() == QtCore.Qt.ControlModifier
+            )
+            oid = self._navigator._current_oid
+            project = oid.split("/")[1]
+            search_oid = "/%s/search" % project
+            self._navigator.session.cmds.Flow.set_value(search_oid + "/query", self.text())
+            self._navigator.goto(search_oid, in_new_view)
+            self.nav_oid_bar.nav_oid.hide()
+            # self.end_edit()
+            # self.set_search_enabled(False)
             return
         self.end_edit()
+        self.set_search_enabled(False)
         in_new_view = (
             QtWidgets.QApplication.keyboardModifiers() == QtCore.Qt.ControlModifier
         )
         self._navigator.goto(oid, in_new_view)
 
     def reject(self):
+        oid = self._navigator._current_oid
+        if oid.endswith('/search'):
+            return
+
         self.reset()
         self.end_edit()
+        self.set_search_enabled(False)
 
     def end_edit(self):
+        # self.completer.model().beginResetModel()
+        self.completer.model().setStringList([])
+        # self.completer.model().endResetModel()
         self.hide()
         self.nav_oid_bar.nav_oid.show()
 
     def focusOutEvent(self, event):
-        super().focusOutEvent(event)
+        super().focusOutEvent(event)       
         self.reject()  # if focus lost, hide automatically
     
     def keyPressEvent(self, e):
@@ -315,11 +387,33 @@ class NavigationOIDField(QtWidgets.QLineEdit):
         return super().keyPressEvent(e)
 
 
+class ProjectSelectionWidget(QtWidgets.QWidget):
+
+    def __init__(self, parent, navigator):
+        super(ProjectSelectionWidget, self).__init__(parent)
+
+        icon = resources.get_icon(('icons.search', 'magn-glass'))
+        self.nav_mode_icon = QtWidgets.QLabel('')
+        self.nav_mode_icon.setPixmap(icon.pixmap(QtCore.QSize(16, 16)))
+        self.nav_project_cbb = QtWidgets.QComboBox()
+        self.nav_project_cbb.addItems(navigator.session.cmds.Search.list_project_names())
+        self.nav_project_cbb.setVisible(self.nav_project_cbb.count() > 1)
+        col = QtWidgets.QApplication.instance().palette().button().color().name(QtGui.QColor.HexRgb)
+        self.nav_project_cbb.setStyleSheet(f'background-color: {col};')
+
+        lo = QtWidgets.QHBoxLayout()
+        lo.setContentsMargins(0, 0, 0, 0)
+        lo.addWidget(self.nav_mode_icon)
+        lo.addWidget(self.nav_project_cbb)
+        self.setLayout(lo)
+
+
 class NavigationOIDBar(QtWidgets.QWidget):
     def __init__(self, parent, navigator):
         super(NavigationOIDBar, self).__init__(parent)
 
         self._navigator = navigator
+        self._search_enabled = False
 
         self.setAttribute(QtCore.Qt.WA_StyledBackground, True)
         self.setMinimumHeight(32)
@@ -327,12 +421,18 @@ class NavigationOIDBar(QtWidgets.QWidget):
 
         self.nav_oid_field = NavigationOIDField(self, navigator)
         self.nav_oid = NavigationOIDControls(self, navigator)
+        self.nav_project_select = ProjectSelectionWidget(self, self._navigator)
+
+        self._select_fx = QtWidgets.QGraphicsOpacityEffect(self.nav_project_select)
+        self._select_fx.setOpacity(0.5)
+        self.nav_project_select.setGraphicsEffect(self._select_fx)
         
         self.nav_oid_field.hide()
         layout.addWidget(self.nav_oid, 100, alignment=QtCore.Qt.AlignVCenter)
         layout.addWidget(self.nav_oid_field, 100, alignment=QtCore.Qt.AlignVCenter)
+        layout.addWidget(self.nav_project_select, 0, alignment=QtCore.Qt.AlignVCenter)
 
-        layout.setMargin(0)
+        layout.setContentsMargins(5, 0, 5, 0)
         self.setLayout(layout)
 
     def setFocusOnOidField(self):
@@ -340,8 +440,19 @@ class NavigationOIDBar(QtWidgets.QWidget):
 
     def mousePressEvent(self, e):
         if e.button() == QtCore.Qt.LeftButton:
+            self.set_search_enabled(True)
             self.setFocusOnOidField()
         super(NavigationOIDBar, self).mousePressEvent(e)
+    
+    def search_enabled(self):
+        return self._search_enabled
+    
+    def set_search_enabled(self, b):
+        self._search_enabled = b
+        self._select_fx.setOpacity(0.5 + 0.5 * float(b))
+    
+    def selected_project(self):
+        return self.nav_project_select.nav_project_cbb.currentText()
 
 
 class NavigationHistoryControls(QtWidgets.QWidget):
@@ -351,64 +462,67 @@ class NavigationHistoryControls(QtWidgets.QWidget):
 
         self._navigator = navigator
 
-        self.prev_bt = QtWidgets.QToolButton(self)
-        self.prev_bt.setProperty('no_border', True)
-        self.prev_bt.setProperty('class', ["nav_bt", "prev_bt"])
-        self.prev_bt.setText('<')
+        self.prev_bt = QtWidgets.QPushButton(self)
+        self.prev_bt.setFixedSize(QtCore.QSize(36,36))
         self.prev_bt.setIcon(resources.get_icon(
-            ('icons.gui', 'chevron-sign-left'),
-            disabled_ref=('icons.gui', 'chevron-sign-left-disabled')
+            ('icons.gui', 'nav-arrow-left'),
+            disabled_ref=('icons.gui', 'nav-arrow-left-disabled')
         ))
         self.prev_bt.clicked.connect(self._on_prev_bt)
 
-        self.up_bt = QtWidgets.QToolButton(self)
-        self.up_bt.setProperty('no_border', True)
-        self.up_bt.setProperty('class', ["nav_bt", "up_bt"])
-        self.up_bt.setText('/\\')
-        self.up_bt.setIcon(resources.get_icon(
-            ('icons.gui', 'chevron-sign-up'),
-            disabled_ref=('icons.gui', 'chevron-sign-up-disabled')
-        ))
-        self.up_bt.clicked.connect(self._on_up_bt)
+        # self.up_bt = QtWidgets.QPushButton(self)
+        # self.up_bt.setProperty('no_border', True)
+        # self.up_bt.setProperty('class', ["nav_bt", "up_bt"])
+        # self.up_bt.setText('/\\')
+        # self.up_bt.setIcon(resources.get_icon(
+        #     ('icons.gui', 'chevron-sign-up'),
+        #     disabled_ref=('icons.gui', 'chevron-sign-up-disabled')
+        # ))
+        # self.up_bt.clicked.connect(self._on_up_bt)
 
-        self.next_bt = QtWidgets.QToolButton(self)
-        self.next_bt.setProperty('no_border', True)
-        self.next_bt.setProperty('class', ["nav_bt", "next_bt"])
-        self.next_bt.setText('>')
+        self.next_bt = QtWidgets.QPushButton(self)
+        self.next_bt.setFixedSize(QtCore.QSize(36,36))
         self.next_bt.setIcon(resources.get_icon(
-            ('icons.gui', 'chevron-sign-to-right'),
-            disabled_ref=('icons.gui', 'chevron-sign-to-right-disabled')
+            ('icons.gui', 'nav-arrow-right'),
+            disabled_ref=('icons.gui', 'nav-arrow-right-disabled')
         ))
         self.next_bt.clicked.connect(self._on_next_bt)
 
-        self.home_bt = QtWidgets.QToolButton(self)
-        self.home_bt.setProperty('no_border', True)
-        self.home_bt.setProperty('class', ["nav_bt", "home_bt"])
-        self.home_bt.setText('/')
-        self.home_bt.setIcon(resources.get_icon(
-            ('icons.gui', 'home'),
-            disabled_ref=('icons.gui', 'home-outline')
-        ))
-        self.home_bt.clicked.connect(self._goto_home)
+        # self.home_bt = QtWidgets.QPushButton(self)
+        # self.home_bt.setProperty('no_border', True)
+        # self.home_bt.setProperty('class', ["nav_bt", "home_bt"])
+        # self.home_bt.setText('/')
+        # self.home_bt.setIcon(resources.get_icon(
+        #     ('icons.gui', 'home'),
+        #     disabled_ref=('icons.gui', 'home-outline')
+        # ))
+        # self.home_bt.clicked.connect(self._goto_home)
 
-        self.refresh_bt = QtWidgets.QToolButton(self)
-        self.refresh_bt.setProperty('no_border', True)
-        self.refresh_bt.setProperty('class', ["nav_bt", "refresh_bt"])
-        self.refresh_bt.setText('/')
+        self.mytask_bt = QtWidgets.QPushButton(self)
+        self.mytask_bt.setFixedSize(QtCore.QSize(36,36))
+        self.mytask_bt.setIcon(resources.get_icon(
+            ('icons.gui', 'mytasks'),
+            disabled_ref=('icons.gui', 'mytasks-disabled')
+        ))
+        self.mytask_bt.clicked.connect(self._goto_mytasks)
+
+        self.refresh_bt = QtWidgets.QPushButton(self)
+        self.refresh_bt.setFixedSize(QtCore.QSize(36,36))
         self.refresh_bt.setIcon(resources.get_icon(
-            ('icons.gui', 'refresh'),
-            disabled_ref=('icons.gui', 'refresh')
+            ('icons.gui', 'nav-arrow-refresh'),
+            disabled_ref=('icons.gui', 'nav-arrow-refresh-disabled')
         ))
         self.refresh_bt.clicked.connect(self._refresh)
 
         bt_lo = QtWidgets.QHBoxLayout()
-        bt_lo.setContentsMargins(12, 0, 12, 0)
+        bt_lo.setContentsMargins(0, 0, 0, 0)
         bt_lo.setSpacing(0)
         bt_lo.addWidget(self.prev_bt)
-        bt_lo.addWidget(self.up_bt)
+        # bt_lo.addWidget(self.up_bt)
         bt_lo.addWidget(self.next_bt)
-        bt_lo.addWidget(self.home_bt)
+        # bt_lo.addWidget(self.home_bt)
         bt_lo.addWidget(self.refresh_bt)
+        bt_lo.addWidget(self.mytask_bt)
 
         self.setLayout(bt_lo)
 
@@ -422,6 +536,15 @@ class NavigationHistoryControls(QtWidgets.QWidget):
         # TODO: handle optional new view
         self._navigator.goto_prev()
 
+    def _goto_mytasks(self):
+        oid = self._navigator._current_oid
+        if oid != "/Home":
+            project = oid.split("/")[1]
+            mytasks_oid = "/%s/mytasks" % project
+            self._navigator.goto(mytasks_oid)
+        else:
+            logging.getLogger('kabaret').log(logging.WARNING, "Cannot open mytasks from /Home ! Implement me")
+
     def _on_up_bt(self):
         # TODO: handle optional new view
         self._navigator.goto_parent()
@@ -432,9 +555,10 @@ class NavigationHistoryControls(QtWidgets.QWidget):
 
     def update_controls(self):
         self.prev_bt.setEnabled(self._navigator.has_prev())
-        self.up_bt.setEnabled(self._navigator.has_parent())
+        # self.up_bt.setEnabled(self._navigator.has_parent())
         self.next_bt.setEnabled(self._navigator.has_next())
-        self.home_bt.setEnabled(self._navigator.current_oid is not None and self._navigator.current_oid() != '/Home')
+        # self.home_bt.setEnabled(self._navigator.current_oid is not None and self._navigator.current_oid() != '/Home')
+        self.mytask_bt.setEnabled(self._navigator.current_oid is not None and self._navigator.current_oid() != '/Home')
 
 
 class NavigationBar(QtWidgets.QWidget):
@@ -442,11 +566,12 @@ class NavigationBar(QtWidgets.QWidget):
     def __init__(self, parent, navigator):
         super(NavigationBar, self).__init__(parent)
         layout = QtWidgets.QHBoxLayout(self)
+        layout.setSpacing(3)
         self._navigator = navigator
         self.nav_ctrl = NavigationHistoryControls(self, navigator)
         self.nav_oid_bar = NavigationOIDBar(self, navigator)
-        layout.addWidget(self.nav_ctrl, 10, alignment=QtCore.Qt.AlignVCenter)
-        layout.addWidget(self.nav_oid_bar, 90, alignment=QtCore.Qt.AlignVCenter)
+        layout.addWidget(self.nav_ctrl, alignment=QtCore.Qt.AlignVCenter)
+        layout.addWidget(self.nav_oid_bar, alignment=QtCore.Qt.AlignVCenter)
         self.setLayout(layout)
         self.setAcceptDrops(True)
         self.setAttribute(QtCore.Qt.WA_StyledBackground, True)
