@@ -206,7 +206,7 @@ class ShotSelector(QtWidgets.QTreeWidget):
         self.session = session
 
         self.setHeaderLabels(self.get_columns())
-        self.header().setSectionResizeMode(0, QtWidgets.QHeaderView.Stretch)
+        self.header().setSectionResizeMode(0, QtWidgets.QHeaderView.ResizeMode.Stretch)
         self.header().setStretchLastSection(False)
         self.setSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
         self.setFixedHeight(350)
@@ -438,7 +438,7 @@ class UploadPlayblastSelector(ShotSelector):
             task_name = self.tasks.child(i).text(0)
             self.task_name_to_index[task_name] = i
         
-        self.header().setSectionResizeMode(QtWidgets.QHeaderView.ResizeToContents)
+        self.header().setSectionResizeMode(QtWidgets.QHeaderView.ResizeMode.ResizeToContents)
         self.header().setStretchLastSection(True)
 
 
